from telethon.tl.types.contacts.link import Link
from telethon.tl.types.contacts.contacts_not_modified import ContactsNotModified
from telethon.tl.types.contacts.contacts import Contacts
from telethon.tl.types.contacts.imported_contacts import ImportedContacts
from telethon.tl.types.contacts.blocked import Blocked
from telethon.tl.types.contacts.blocked_slice import BlockedSlice
from telethon.tl.types.contacts.found import Found
from telethon.tl.types.contacts.resolved_peer import ResolvedPeer
from telethon.tl.types.contacts.top_peers_not_modified import TopPeersNotModified
from telethon.tl.types.contacts.top_peers import TopPeers



from telethon.tl.mtproto_request import MTProtoRequest


class MaskCoords(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    maskCoords#aed6dbb2 n:int x:double y:double zoom:double = MaskCoords"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xaed6dbb2

    def __init__(self, n, x, y, zoom):
        """
        :param n: Telegram type: «int».
        :param x: Telegram type: «double».
        :param y: Telegram type: «double».
        :param zoom: Telegram type: «double».
        """
        super().__init__()

        self.n = n
        self.x = x
        self.y = y
        self.zoom = zoom

    def on_send(self, writer):
        writer.write_int(MaskCoords.constructor_id, signed=False)
        writer.write_int(self.n)
        writer.write_double(self.x)
        writer.write_double(self.y)
        writer.write_double(self.zoom)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return MaskCoords(None, None, None, None)

    def on_response(self, reader):
        self.n = reader.read_int()
        self.x = reader.read_double()
        self.y = reader.read_double()
        self.zoom = reader.read_double()

    def __repr__(self):
        return 'maskCoords#aed6dbb2 n:int x:double y:double zoom:double = MaskCoords'

    def __str__(self):
        return '(maskCoords (ID: 0xaed6dbb2) = (n={}, x={}, y={}, zoom={}))'.format(str(self.n), str(self.x), str(self.y), str(self.zoom))



from telethon.tl.mtproto_request import MTProtoRequest


class SaveDeveloperInfoRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    contest.saveDeveloperInfo#9a5f6e95 vk_id:int name:string phone_number:string age:int city:string = Bool"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x9a5f6e95

    def __init__(self, vk_id, name, phone_number, age, city):
        """
        :param vk_id: Telegram type: «int».
        :param name: Telegram type: «string».
        :param phone_number: Telegram type: «string».
        :param age: Telegram type: «int».
        :param city: Telegram type: «string».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.vk_id = vk_id
        self.name = name
        self.phone_number = phone_number
        self.age = age
        self.city = city

    def on_send(self, writer):
        writer.write_int(SaveDeveloperInfoRequest.constructor_id, signed=False)
        writer.write_int(self.vk_id)
        writer.tgwrite_string(self.name)
        writer.tgwrite_string(self.phone_number)
        writer.write_int(self.age)
        writer.tgwrite_string(self.city)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return SaveDeveloperInfoRequest(None, None, None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'contest.saveDeveloperInfo#9a5f6e95 vk_id:int name:string phone_number:string age:int city:string = Bool'

    def __str__(self):
        return '(contest.saveDeveloperInfo (ID: 0x9a5f6e95) = (vk_id={}, name={}, phone_number={}, age={}, city={}))'.format(str(self.vk_id), str(self.name), str(self.phone_number), str(self.age), str(self.city))



from telethon.tl.mtproto_request import MTProtoRequest


class AccountDaysTTL(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    accountDaysTTL#b8d0afdf days:int = AccountDaysTTL"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xb8d0afdf

    def __init__(self, days):
        """
        :param days: Telegram type: «int».
        """
        super().__init__()

        self.days = days

    def on_send(self, writer):
        writer.write_int(AccountDaysTTL.constructor_id, signed=False)
        writer.write_int(self.days)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return AccountDaysTTL(None)

    def on_response(self, reader):
        self.days = reader.read_int()

    def __repr__(self):
        return 'accountDaysTTL#b8d0afdf days:int = AccountDaysTTL'

    def __str__(self):
        return '(accountDaysTTL (ID: 0xb8d0afdf) = (days={}))'.format(str(self.days))



from telethon.tl.mtproto_request import MTProtoRequest


class RpcError(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    rpc_error#2144ca19 error_code:int error_message:string = RpcError"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x2144ca19

    def __init__(self, error_code, error_message):
        """
        :param error_code: Telegram type: «int».
        :param error_message: Telegram type: «string».
        """
        super().__init__()

        self.error_code = error_code
        self.error_message = error_message

    def on_send(self, writer):
        writer.write_int(RpcError.constructor_id, signed=False)
        writer.write_int(self.error_code)
        writer.tgwrite_string(self.error_message)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return RpcError(None, None)

    def on_response(self, reader):
        self.error_code = reader.read_int()
        self.error_message = reader.tgread_string()

    def __repr__(self):
        return 'rpc_error#2144ca19 error_code:int error_message:string = RpcError'

    def __str__(self):
        return '(rpc_error (ID: 0x2144ca19) = (error_code={}, error_message={}))'.format(str(self.error_code), str(self.error_message))



from telethon.tl.mtproto_request import MTProtoRequest


class BadMsgNotification(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    bad_msg_notification#a7eff811 bad_msg_id:long bad_msg_seqno:int error_code:int = BadMsgNotification"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xa7eff811

    def __init__(self, bad_msg_id, bad_msg_seqno, error_code):
        """
        :param bad_msg_id: Telegram type: «long».
        :param bad_msg_seqno: Telegram type: «int».
        :param error_code: Telegram type: «int».
        """
        super().__init__()

        self.bad_msg_id = bad_msg_id
        self.bad_msg_seqno = bad_msg_seqno
        self.error_code = error_code

    def on_send(self, writer):
        writer.write_int(BadMsgNotification.constructor_id, signed=False)
        writer.write_long(self.bad_msg_id)
        writer.write_int(self.bad_msg_seqno)
        writer.write_int(self.error_code)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return BadMsgNotification(None, None, None)

    def on_response(self, reader):
        self.bad_msg_id = reader.read_long()
        self.bad_msg_seqno = reader.read_int()
        self.error_code = reader.read_int()

    def __repr__(self):
        return 'bad_msg_notification#a7eff811 bad_msg_id:long bad_msg_seqno:int error_code:int = BadMsgNotification'

    def __str__(self):
        return '(bad_msg_notification (ID: 0xa7eff811) = (bad_msg_id={}, bad_msg_seqno={}, error_code={}))'.format(str(self.bad_msg_id), str(self.bad_msg_seqno), str(self.error_code))



from telethon.tl.types.updates.state import State
from telethon.tl.types.updates.difference_empty import DifferenceEmpty
from telethon.tl.types.updates.difference import Difference
from telethon.tl.types.updates.difference_slice import DifferenceSlice
from telethon.tl.types.updates.channel_difference_empty import ChannelDifferenceEmpty
from telethon.tl.types.updates.channel_difference_too_long import ChannelDifferenceTooLong
from telethon.tl.types.updates.channel_difference import ChannelDifference



from telethon.tl.mtproto_request import MTProtoRequest


class HttpWait(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    http_wait#9299359f max_delay:int wait_after:int max_wait:int = HttpWait"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x9299359f

    def __init__(self, max_delay, wait_after, max_wait):
        """
        :param max_delay: Telegram type: «int».
        :param wait_after: Telegram type: «int».
        :param max_wait: Telegram type: «int».
        """
        super().__init__()

        self.max_delay = max_delay
        self.wait_after = wait_after
        self.max_wait = max_wait

    def on_send(self, writer):
        writer.write_int(HttpWait.constructor_id, signed=False)
        writer.write_int(self.max_delay)
        writer.write_int(self.wait_after)
        writer.write_int(self.max_wait)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return HttpWait(None, None, None)

    def on_response(self, reader):
        self.max_delay = reader.read_int()
        self.wait_after = reader.read_int()
        self.max_wait = reader.read_int()

    def __repr__(self):
        return 'http_wait#9299359f max_delay:int wait_after:int max_wait:int = HttpWait'

    def __str__(self):
        return '(http_wait (ID: 0x9299359f) = (max_delay={}, wait_after={}, max_wait={}))'.format(str(self.max_delay), str(self.wait_after), str(self.max_wait))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdatesTg(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updates_tg#74ae4240 updates:Vector<Update> users:Vector<User> chats:Vector<Chat> date:date seq:int = Updates"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x74ae4240

    def __init__(self, updates, users, chats, date, seq):
        """
        :param updates: Telegram type: «Update». Must be a list.
        :param users: Telegram type: «User». Must be a list.
        :param chats: Telegram type: «Chat». Must be a list.
        :param date: Telegram type: «date».
        :param seq: Telegram type: «int».
        """
        super().__init__()

        self.updates = updates
        self.users = users
        self.chats = chats
        self.date = date
        self.seq = seq

    def on_send(self, writer):
        writer.write_int(UpdatesTg.constructor_id, signed=False)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.updates))
        for updates_item in self.updates:
            updates_item.on_send(writer)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.users))
        for users_item in self.users:
            users_item.on_send(writer)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.chats))
        for chats_item in self.chats:
            chats_item.on_send(writer)

        writer.tgwrite_date(self.date)
        writer.write_int(self.seq)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdatesTg(None, None, None, None, None)

    def on_response(self, reader):
        reader.read_int()  # Vector's constructor ID
        self.updates = []  # Initialize an empty list
        updates_len = reader.read_int()
        for _ in range(updates_len):
            updates_item = reader.tgread_object()
            self.updates.append(updates_item)

        reader.read_int()  # Vector's constructor ID
        self.users = []  # Initialize an empty list
        users_len = reader.read_int()
        for _ in range(users_len):
            users_item = reader.tgread_object()
            self.users.append(users_item)

        reader.read_int()  # Vector's constructor ID
        self.chats = []  # Initialize an empty list
        chats_len = reader.read_int()
        for _ in range(chats_len):
            chats_item = reader.tgread_object()
            self.chats.append(chats_item)

        self.date = reader.tgread_date()
        self.seq = reader.read_int()

    def __repr__(self):
        return 'updates_tg#74ae4240 updates:Vector<Update> users:Vector<User> chats:Vector<Chat> date:date seq:int = Updates'

    def __str__(self):
        return '(updates_tg (ID: 0x74ae4240) = (updates={}, users={}, chats={}, date={}, seq={}))'.format(None if not self.updates else [str(_) for _ in self.updates], None if not self.users else [str(_) for _ in self.users], None if not self.chats else [str(_) for _ in self.chats], str(self.date), str(self.seq))



from telethon.tl.mtproto_request import MTProtoRequest


class GetInlineBotResultsRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.getInlineBotResults#514e999d flags:None bot:InputUser peer:InputPeer geo_point:flags.0?InputGeoPoint query:string offset:string = messages.BotResults"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x514e999d

    def __init__(self, bot, peer, query, offset, geo_point=None):
        """
        :param bot: Telegram type: «InputUser».
        :param peer: Telegram type: «InputPeer».
        :param geo_point: Telegram type: «InputGeoPoint».
        :param query: Telegram type: «string».
        :param offset: Telegram type: «string».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.bot = bot
        self.peer = peer
        self.geo_point = geo_point
        self.query = query
        self.offset = offset

    def on_send(self, writer):
        writer.write_int(GetInlineBotResultsRequest.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.geo_point else 0
        writer.write_int(flags)

        self.bot.on_send(writer)
        self.peer.on_send(writer)
        if self.geo_point:
            self.geo_point.on_send(writer)

        writer.tgwrite_string(self.query)
        writer.tgwrite_string(self.offset)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return GetInlineBotResultsRequest(None, None, None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.getInlineBotResults#514e999d flags:None bot:InputUser peer:InputPeer geo_point:flags.0?InputGeoPoint query:string offset:string = messages.BotResults'

    def __str__(self):
        return '(messages.getInlineBotResults (ID: 0x514e999d) = (bot={}, peer={}, geo_point={}, query={}, offset={}))'.format(str(self.bot), str(self.peer), str(self.geo_point), str(self.query), str(self.offset))



from telethon.tl.mtproto_request import MTProtoRequest


class InitConnectionRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    initConnection#69796de9 {X:Type} api_id:int device_model:string system_version:string app_version:string lang_code:string query:!X = X"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x69796de9

    def __init__(self, api_id, device_model, system_version, app_version, lang_code, query):
        """
        :param api_id: Telegram type: «int».
        :param device_model: Telegram type: «string».
        :param system_version: Telegram type: «string».
        :param app_version: Telegram type: «string».
        :param lang_code: Telegram type: «string».
        :param query: Telegram type: «X». This should be another MTProtoRequest.
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.api_id = api_id
        self.device_model = device_model
        self.system_version = system_version
        self.app_version = app_version
        self.lang_code = lang_code
        self.query = query

    def on_send(self, writer):
        writer.write_int(InitConnectionRequest.constructor_id, signed=False)
        writer.write_int(self.api_id)
        writer.tgwrite_string(self.device_model)
        writer.tgwrite_string(self.system_version)
        writer.tgwrite_string(self.app_version)
        writer.tgwrite_string(self.lang_code)
        self.query.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InitConnectionRequest(None, None, None, None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'initConnection#69796de9 {X:Type} api_id:int device_model:string system_version:string app_version:string lang_code:string query:!X = X'

    def __str__(self):
        return '(initConnection (ID: 0x69796de9) = (api_id={}, device_model={}, system_version={}, app_version={}, lang_code={}, query={}))'.format(str(self.api_id), str(self.device_model), str(self.system_version), str(self.app_version), str(self.lang_code), str(self.query))



from telethon.tl.mtproto_request import MTProtoRequest


class DhConfig(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.dhConfig#2c221edd g:int p:bytes version:int random:bytes = messages.DhConfig"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x2c221edd

    def __init__(self, g, p, version, random):
        """
        :param g: Telegram type: «int».
        :param p: Telegram type: «bytes».
        :param version: Telegram type: «int».
        :param random: Telegram type: «bytes».
        """
        super().__init__()

        self.g = g
        self.p = p
        self.version = version
        self.random = random

    def on_send(self, writer):
        writer.write_int(DhConfig.constructor_id, signed=False)
        writer.write_int(self.g)
        writer.tgwrite_bytes(self.p)
        writer.write_int(self.version)
        writer.tgwrite_bytes(self.random)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return DhConfig(None, None, None, None)

    def on_response(self, reader):
        self.g = reader.read_int()
        self.p = reader.tgread_bytes()
        self.version = reader.read_int()
        self.random = reader.tgread_bytes()

    def __repr__(self):
        return 'messages.dhConfig#2c221edd g:int p:bytes version:int random:bytes = messages.DhConfig'

    def __str__(self):
        return '(messages.dhConfig (ID: 0x2c221edd) = (g={}, p={}, version={}, random={}))'.format(str(self.g), str(self.p), str(self.version), str(self.random))



from telethon.tl.mtproto_request import MTProtoRequest


class InputStickerSetID(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    inputStickerSetID#9de7a269 id:long access_hash:long = InputStickerSet"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x9de7a269

    def __init__(self, id, access_hash):
        """
        :param id: Telegram type: «long».
        :param access_hash: Telegram type: «long».
        """
        super().__init__()

        self.id = id
        self.access_hash = access_hash

    def on_send(self, writer):
        writer.write_int(InputStickerSetID.constructor_id, signed=False)
        writer.write_long(self.id)
        writer.write_long(self.access_hash)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InputStickerSetID(None, None)

    def on_response(self, reader):
        self.id = reader.read_long()
        self.access_hash = reader.read_long()

    def __repr__(self):
        return 'inputStickerSetID#9de7a269 id:long access_hash:long = InputStickerSet'

    def __str__(self):
        return '(inputStickerSetID (ID: 0x9de7a269) = (id={}, access_hash={}))'.format(str(self.id), str(self.access_hash))



from telethon.tl.mtproto_request import MTProtoRequest


class Password(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    account.password#7c18141c current_salt:bytes new_salt:bytes hint:string has_recovery:Bool email_unconfirmed_pattern:string = account.Password"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x7c18141c

    def __init__(self, current_salt, new_salt, hint, has_recovery, email_unconfirmed_pattern):
        """
        :param current_salt: Telegram type: «bytes».
        :param new_salt: Telegram type: «bytes».
        :param hint: Telegram type: «string».
        :param has_recovery: Telegram type: «Bool».
        :param email_unconfirmed_pattern: Telegram type: «string».
        """
        super().__init__()

        self.current_salt = current_salt
        self.new_salt = new_salt
        self.hint = hint
        self.has_recovery = has_recovery
        self.email_unconfirmed_pattern = email_unconfirmed_pattern

    def on_send(self, writer):
        writer.write_int(Password.constructor_id, signed=False)
        writer.tgwrite_bytes(self.current_salt)
        writer.tgwrite_bytes(self.new_salt)
        writer.tgwrite_string(self.hint)
        writer.tgwrite_bool(self.has_recovery)
        writer.tgwrite_string(self.email_unconfirmed_pattern)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return Password(None, None, None, None, None)

    def on_response(self, reader):
        self.current_salt = reader.tgread_bytes()
        self.new_salt = reader.tgread_bytes()
        self.hint = reader.tgread_string()
        self.has_recovery = reader.tgread_bool()
        self.email_unconfirmed_pattern = reader.tgread_string()

    def __repr__(self):
        return 'account.password#7c18141c current_salt:bytes new_salt:bytes hint:string has_recovery:Bool email_unconfirmed_pattern:string = account.Password'

    def __str__(self):
        return '(account.password (ID: 0x7c18141c) = (current_salt={}, new_salt={}, hint={}, has_recovery={}, email_unconfirmed_pattern={}))'.format(str(self.current_salt), str(self.new_salt), str(self.hint), str(self.has_recovery), str(self.email_unconfirmed_pattern))



from telethon.tl.types.messages.dialogs import Dialogs
from telethon.tl.types.messages.dialogs_slice import DialogsSlice
from telethon.tl.types.messages.messages import Messages
from telethon.tl.types.messages.messages_slice import MessagesSlice
from telethon.tl.types.messages.channel_messages import ChannelMessages
from telethon.tl.types.messages.chats import Chats
from telethon.tl.types.messages.chat_full import ChatFull
from telethon.tl.types.messages.affected_history import AffectedHistory
from telethon.tl.types.messages.dh_config_not_modified import DhConfigNotModified
from telethon.tl.types.messages.dh_config import DhConfig
from telethon.tl.types.messages.sent_encrypted_message import SentEncryptedMessage
from telethon.tl.types.messages.sent_encrypted_file import SentEncryptedFile
from telethon.tl.types.messages.stickers_not_modified import StickersNotModified
from telethon.tl.types.messages.stickers import Stickers
from telethon.tl.types.messages.all_stickers_not_modified import AllStickersNotModified
from telethon.tl.types.messages.all_stickers import AllStickers
from telethon.tl.types.messages.affected_messages import AffectedMessages
from telethon.tl.types.messages.sticker_set import StickerSet
from telethon.tl.types.messages.found_gifs import FoundGifs
from telethon.tl.types.messages.saved_gifs_not_modified import SavedGifsNotModified
from telethon.tl.types.messages.saved_gifs import SavedGifs
from telethon.tl.types.messages.bot_results import BotResults
from telethon.tl.types.messages.bot_callback_answer import BotCallbackAnswer
from telethon.tl.types.messages.message_edit_data import MessageEditData
from telethon.tl.types.messages.peer_dialogs import PeerDialogs
from telethon.tl.types.messages.featured_stickers_not_modified import FeaturedStickersNotModified
from telethon.tl.types.messages.featured_stickers import FeaturedStickers
from telethon.tl.types.messages.recent_stickers_not_modified import RecentStickersNotModified
from telethon.tl.types.messages.recent_stickers import RecentStickers
from telethon.tl.types.messages.archived_stickers import ArchivedStickers
from telethon.tl.types.messages.sticker_set_install_result_success import StickerSetInstallResultSuccess
from telethon.tl.types.messages.sticker_set_install_result_archive import StickerSetInstallResultArchive
from telethon.tl.types.messages.high_scores import HighScores



from telethon.tl.mtproto_request import MTProtoRequest


class RecoverPasswordRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    auth.recoverPassword#4ea56e92 code:string = auth.Authorization"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x4ea56e92

    def __init__(self, code):
        """
        :param code: Telegram type: «string».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.code = code

    def on_send(self, writer):
        writer.write_int(RecoverPasswordRequest.constructor_id, signed=False)
        writer.tgwrite_string(self.code)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return RecoverPasswordRequest(None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'auth.recoverPassword#4ea56e92 code:string = auth.Authorization'

    def __str__(self):
        return '(auth.recoverPassword (ID: 0x4ea56e92) = (code={}))'.format(str(self.code))



from telethon.tl.mtproto_request import MTProtoRequest


class RpcDropAnswerRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    rpc_drop_answer#58e4a740 req_msg_id:long = RpcDropAnswer"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x58e4a740

    def __init__(self, req_msg_id):
        """
        :param req_msg_id: Telegram type: «long».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.req_msg_id = req_msg_id

    def on_send(self, writer):
        writer.write_int(RpcDropAnswerRequest.constructor_id, signed=False)
        writer.write_long(self.req_msg_id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return RpcDropAnswerRequest(None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'rpc_drop_answer#58e4a740 req_msg_id:long = RpcDropAnswer'

    def __str__(self):
        return '(rpc_drop_answer (ID: 0x58e4a740) = (req_msg_id={}))'.format(str(self.req_msg_id))



from telethon.tl.mtproto_request import MTProtoRequest


class ClientDHInnerData(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    client_DH_inner_data#6643b654 nonce:int128 server_nonce:int128 retry_id:long g_b:string = Client_DH_Inner_Data"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x6643b654

    def __init__(self, nonce, server_nonce, retry_id, g_b):
        """
        :param nonce: Telegram type: «int128».
        :param server_nonce: Telegram type: «int128».
        :param retry_id: Telegram type: «long».
        :param g_b: Telegram type: «string».
        """
        super().__init__()

        self.nonce = nonce
        self.server_nonce = server_nonce
        self.retry_id = retry_id
        self.g_b = g_b

    def on_send(self, writer):
        writer.write_int(ClientDHInnerData.constructor_id, signed=False)
        writer.write_large_int(self.nonce, bits=128)
        writer.write_large_int(self.server_nonce, bits=128)
        writer.write_long(self.retry_id)
        writer.tgwrite_string(self.g_b)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ClientDHInnerData(None, None, None, None)

    def on_response(self, reader):
        self.nonce = reader.read_large_int(bits=128)
        self.server_nonce = reader.read_large_int(bits=128)
        self.retry_id = reader.read_long()
        self.g_b = reader.tgread_string()

    def __repr__(self):
        return 'client_DH_inner_data#6643b654 nonce:int128 server_nonce:int128 retry_id:long g_b:string = Client_DH_Inner_Data'

    def __str__(self):
        return '(client_DH_inner_data (ID: 0x6643b654) = (nonce={}, server_nonce={}, retry_id={}, g_b={}))'.format(str(self.nonce), str(self.server_nonce), str(self.retry_id), str(self.g_b))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateDcOptions(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateDcOptions#8e5e9873 dc_options:Vector<DcOption> = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x8e5e9873

    def __init__(self, dc_options):
        """
        :param dc_options: Telegram type: «DcOption». Must be a list.
        """
        super().__init__()

        self.dc_options = dc_options

    def on_send(self, writer):
        writer.write_int(UpdateDcOptions.constructor_id, signed=False)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.dc_options))
        for dc_options_item in self.dc_options:
            dc_options_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateDcOptions(None)

    def on_response(self, reader):
        reader.read_int()  # Vector's constructor ID
        self.dc_options = []  # Initialize an empty list
        dc_options_len = reader.read_int()
        for _ in range(dc_options_len):
            dc_options_item = reader.tgread_object()
            self.dc_options.append(dc_options_item)

    def __repr__(self):
        return 'updateDcOptions#8e5e9873 dc_options:Vector<DcOption> = Update'

    def __str__(self):
        return '(updateDcOptions (ID: 0x8e5e9873) = (dc_options={}))'.format(None if not self.dc_options else [str(_) for _ in self.dc_options])



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateDraftMessage(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateDraftMessage#ee2bb969 peer:Peer draft:DraftMessage = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xee2bb969

    def __init__(self, peer, draft):
        """
        :param peer: Telegram type: «Peer».
        :param draft: Telegram type: «DraftMessage».
        """
        super().__init__()

        self.peer = peer
        self.draft = draft

    def on_send(self, writer):
        writer.write_int(UpdateDraftMessage.constructor_id, signed=False)
        self.peer.on_send(writer)
        self.draft.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateDraftMessage(None, None)

    def on_response(self, reader):
        self.peer = reader.tgread_object()
        self.draft = reader.tgread_object()

    def __repr__(self):
        return 'updateDraftMessage#ee2bb969 peer:Peer draft:DraftMessage = Update'

    def __str__(self):
        return '(updateDraftMessage (ID: 0xee2bb969) = (peer={}, draft={}))'.format(str(self.peer), str(self.draft))



from telethon.tl.mtproto_request import MTProtoRequest


class KeyboardButtonSwitchInline(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    keyboardButtonSwitchInline#0568a748 flags:None same_peer:flags.0?true text:string query:string = KeyboardButton"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x568a748

    def __init__(self, text, query, same_peer=None):
        """
        :param same_peer: Telegram type: «true».
        :param text: Telegram type: «string».
        :param query: Telegram type: «string».
        """
        super().__init__()

        self.same_peer = same_peer
        self.text = text
        self.query = query

    def on_send(self, writer):
        writer.write_int(KeyboardButtonSwitchInline.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.same_peer else 0
        writer.write_int(flags)

        writer.tgwrite_string(self.text)
        writer.tgwrite_string(self.query)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return KeyboardButtonSwitchInline(None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 0)) != 0:
            self.same_peer = True  # Arbitrary not-None value, no need to read since it is a flag

        self.text = reader.tgread_string()
        self.query = reader.tgread_string()

    def __repr__(self):
        return 'keyboardButtonSwitchInline#0568a748 flags:None same_peer:flags.0?true text:string query:string = KeyboardButton'

    def __str__(self):
        return '(keyboardButtonSwitchInline (ID: 0x568a748) = (same_peer={}, text={}, query={}))'.format(str(self.same_peer), str(self.text), str(self.query))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateUserPhone(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateUserPhone#12b9417b user_id:int phone:string = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x12b9417b

    def __init__(self, user_id, phone):
        """
        :param user_id: Telegram type: «int».
        :param phone: Telegram type: «string».
        """
        super().__init__()

        self.user_id = user_id
        self.phone = phone

    def on_send(self, writer):
        writer.write_int(UpdateUserPhone.constructor_id, signed=False)
        writer.write_int(self.user_id)
        writer.tgwrite_string(self.phone)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateUserPhone(None, None)

    def on_response(self, reader):
        self.user_id = reader.read_int()
        self.phone = reader.tgread_string()

    def __repr__(self):
        return 'updateUserPhone#12b9417b user_id:int phone:string = Update'

    def __str__(self):
        return '(updateUserPhone (ID: 0x12b9417b) = (user_id={}, phone={}))'.format(str(self.user_id), str(self.phone))



from telethon.tl.mtproto_request import MTProtoRequest


class MessageActionChannelCreate(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messageActionChannelCreate#95d2ac92 title:string = MessageAction"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x95d2ac92

    def __init__(self, title):
        """
        :param title: Telegram type: «string».
        """
        super().__init__()

        self.title = title

    def on_send(self, writer):
        writer.write_int(MessageActionChannelCreate.constructor_id, signed=False)
        writer.tgwrite_string(self.title)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return MessageActionChannelCreate(None)

    def on_response(self, reader):
        self.title = reader.tgread_string()

    def __repr__(self):
        return 'messageActionChannelCreate#95d2ac92 title:string = MessageAction'

    def __str__(self):
        return '(messageActionChannelCreate (ID: 0x95d2ac92) = (title={}))'.format(str(self.title))



from telethon.tl.mtproto_request import MTProtoRequest


class MessageFwdHeader(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messageFwdHeader#c786ddcb flags:None from_id:flags.0?int date:date channel_id:flags.1?int channel_post:flags.2?int = MessageFwdHeader"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xc786ddcb

    def __init__(self, date, from_id=None, channel_id=None, channel_post=None):
        """
        :param from_id: Telegram type: «int».
        :param date: Telegram type: «date».
        :param channel_id: Telegram type: «int».
        :param channel_post: Telegram type: «int».
        """
        super().__init__()

        self.from_id = from_id
        self.date = date
        self.channel_id = channel_id
        self.channel_post = channel_post

    def on_send(self, writer):
        writer.write_int(MessageFwdHeader.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.from_id else 0
        flags |= (1 << 1) if self.channel_id else 0
        flags |= (1 << 2) if self.channel_post else 0
        writer.write_int(flags)

        if self.from_id:
            writer.write_int(self.from_id)

        writer.tgwrite_date(self.date)
        if self.channel_id:
            writer.write_int(self.channel_id)

        if self.channel_post:
            writer.write_int(self.channel_post)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return MessageFwdHeader(None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 0)) != 0:
            self.from_id = reader.read_int()

        self.date = reader.tgread_date()
        if (flags & (1 << 1)) != 0:
            self.channel_id = reader.read_int()

        if (flags & (1 << 2)) != 0:
            self.channel_post = reader.read_int()

    def __repr__(self):
        return 'messageFwdHeader#c786ddcb flags:None from_id:flags.0?int date:date channel_id:flags.1?int channel_post:flags.2?int = MessageFwdHeader'

    def __str__(self):
        return '(messageFwdHeader (ID: 0xc786ddcb) = (from_id={}, date={}, channel_id={}, channel_post={}))'.format(str(self.from_id), str(self.date), str(self.channel_id), str(self.channel_post))



from telethon.tl.mtproto_request import MTProtoRequest


class GetArchivedStickersRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.getArchivedStickers#57f17692 flags:None masks:flags.0?true offset_id:long limit:int = messages.ArchivedStickers"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x57f17692

    def __init__(self, offset_id, limit, masks=None):
        """
        :param masks: Telegram type: «true».
        :param offset_id: Telegram type: «long».
        :param limit: Telegram type: «int».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.masks = masks
        self.offset_id = offset_id
        self.limit = limit

    def on_send(self, writer):
        writer.write_int(GetArchivedStickersRequest.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.masks else 0
        writer.write_int(flags)

        writer.write_long(self.offset_id)
        writer.write_int(self.limit)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return GetArchivedStickersRequest(None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.getArchivedStickers#57f17692 flags:None masks:flags.0?true offset_id:long limit:int = messages.ArchivedStickers'

    def __str__(self):
        return '(messages.getArchivedStickers (ID: 0x57f17692) = (masks={}, offset_id={}, limit={}))'.format(str(self.masks), str(self.offset_id), str(self.limit))



from telethon.tl.mtproto_request import MTProtoRequest


class InputGameShortName(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    inputGameShortName#c331e80a bot_id:InputUser short_name:string = InputGame"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xc331e80a

    def __init__(self, bot_id, short_name):
        """
        :param bot_id: Telegram type: «InputUser».
        :param short_name: Telegram type: «string».
        """
        super().__init__()

        self.bot_id = bot_id
        self.short_name = short_name

    def on_send(self, writer):
        writer.write_int(InputGameShortName.constructor_id, signed=False)
        self.bot_id.on_send(writer)
        writer.tgwrite_string(self.short_name)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InputGameShortName(None, None)

    def on_response(self, reader):
        self.bot_id = reader.tgread_object()
        self.short_name = reader.tgread_string()

    def __repr__(self):
        return 'inputGameShortName#c331e80a bot_id:InputUser short_name:string = InputGame'

    def __str__(self):
        return '(inputGameShortName (ID: 0xc331e80a) = (bot_id={}, short_name={}))'.format(str(self.bot_id), str(self.short_name))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateChatParticipantDelete(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateChatParticipantDelete#6e5f8c22 chat_id:int user_id:int version:int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x6e5f8c22

    def __init__(self, chat_id, user_id, version):
        """
        :param chat_id: Telegram type: «int».
        :param user_id: Telegram type: «int».
        :param version: Telegram type: «int».
        """
        super().__init__()

        self.chat_id = chat_id
        self.user_id = user_id
        self.version = version

    def on_send(self, writer):
        writer.write_int(UpdateChatParticipantDelete.constructor_id, signed=False)
        writer.write_int(self.chat_id)
        writer.write_int(self.user_id)
        writer.write_int(self.version)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateChatParticipantDelete(None, None, None)

    def on_response(self, reader):
        self.chat_id = reader.read_int()
        self.user_id = reader.read_int()
        self.version = reader.read_int()

    def __repr__(self):
        return 'updateChatParticipantDelete#6e5f8c22 chat_id:int user_id:int version:int = Update'

    def __str__(self):
        return '(updateChatParticipantDelete (ID: 0x6e5f8c22) = (chat_id={}, user_id={}, version={}))'.format(str(self.chat_id), str(self.user_id), str(self.version))



from telethon.tl.mtproto_request import MTProtoRequest


class ChatParticipantCreator(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    chatParticipantCreator#da13538a user_id:int = ChatParticipant"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xda13538a

    def __init__(self, user_id):
        """
        :param user_id: Telegram type: «int».
        """
        super().__init__()

        self.user_id = user_id

    def on_send(self, writer):
        writer.write_int(ChatParticipantCreator.constructor_id, signed=False)
        writer.write_int(self.user_id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ChatParticipantCreator(None)

    def on_response(self, reader):
        self.user_id = reader.read_int()

    def __repr__(self):
        return 'chatParticipantCreator#da13538a user_id:int = ChatParticipant'

    def __str__(self):
        return '(chatParticipantCreator (ID: 0xda13538a) = (user_id={}))'.format(str(self.user_id))



from telethon.tl.functions.help.get_config import GetConfigRequest
from telethon.tl.functions.help.get_nearest_dc import GetNearestDcRequest
from telethon.tl.functions.help.get_app_update import GetAppUpdateRequest
from telethon.tl.functions.help.save_app_log import SaveAppLogRequest
from telethon.tl.functions.help.get_invite_text import GetInviteTextRequest
from telethon.tl.functions.help.get_support import GetSupportRequest
from telethon.tl.functions.help.get_app_changelog import GetAppChangelogRequest
from telethon.tl.functions.help.get_terms_of_service import GetTermsOfServiceRequest



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateBotInlineQuery(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateBotInlineQuery#54826690 flags:None query_id:long user_id:int query:string geo:flags.0?GeoPoint offset:string = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x54826690

    def __init__(self, query_id, user_id, query, offset, geo=None):
        """
        :param query_id: Telegram type: «long».
        :param user_id: Telegram type: «int».
        :param query: Telegram type: «string».
        :param geo: Telegram type: «GeoPoint».
        :param offset: Telegram type: «string».
        """
        super().__init__()

        self.query_id = query_id
        self.user_id = user_id
        self.query = query
        self.geo = geo
        self.offset = offset

    def on_send(self, writer):
        writer.write_int(UpdateBotInlineQuery.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.geo else 0
        writer.write_int(flags)

        writer.write_long(self.query_id)
        writer.write_int(self.user_id)
        writer.tgwrite_string(self.query)
        if self.geo:
            self.geo.on_send(writer)

        writer.tgwrite_string(self.offset)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateBotInlineQuery(None, None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        self.query_id = reader.read_long()
        self.user_id = reader.read_int()
        self.query = reader.tgread_string()
        if (flags & (1 << 0)) != 0:
            self.geo = reader.tgread_object()

        self.offset = reader.tgread_string()

    def __repr__(self):
        return 'updateBotInlineQuery#54826690 flags:None query_id:long user_id:int query:string geo:flags.0?GeoPoint offset:string = Update'

    def __str__(self):
        return '(updateBotInlineQuery (ID: 0x54826690) = (query_id={}, user_id={}, query={}, geo={}, offset={}))'.format(str(self.query_id), str(self.user_id), str(self.query), str(self.geo), str(self.offset))



from telethon.tl.mtproto_request import MTProtoRequest


class ResPQ(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    resPQ#05162463 nonce:int128 server_nonce:int128 pq:string server_public_key_fingerprints:Vector<long> = ResPQ"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x5162463

    def __init__(self, nonce, server_nonce, pq, server_public_key_fingerprints):
        """
        :param nonce: Telegram type: «int128».
        :param server_nonce: Telegram type: «int128».
        :param pq: Telegram type: «string».
        :param server_public_key_fingerprints: Telegram type: «long». Must be a list.
        """
        super().__init__()

        self.nonce = nonce
        self.server_nonce = server_nonce
        self.pq = pq
        self.server_public_key_fingerprints = server_public_key_fingerprints

    def on_send(self, writer):
        writer.write_int(ResPQ.constructor_id, signed=False)
        writer.write_large_int(self.nonce, bits=128)
        writer.write_large_int(self.server_nonce, bits=128)
        writer.tgwrite_string(self.pq)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.server_public_key_fingerprints))
        for server_public_key_fingerprints_item in self.server_public_key_fingerprints:
            writer.write_long(server_public_key_fingerprints_item)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ResPQ(None, None, None, None)

    def on_response(self, reader):
        self.nonce = reader.read_large_int(bits=128)
        self.server_nonce = reader.read_large_int(bits=128)
        self.pq = reader.tgread_string()
        reader.read_int()  # Vector's constructor ID
        self.server_public_key_fingerprints = []  # Initialize an empty list
        server_public_key_fingerprints_len = reader.read_int()
        for _ in range(server_public_key_fingerprints_len):
            server_public_key_fingerprints_item = reader.read_long()
            self.server_public_key_fingerprints.append(server_public_key_fingerprints_item)

    def __repr__(self):
        return 'resPQ#05162463 nonce:int128 server_nonce:int128 pq:string server_public_key_fingerprints:Vector<long> = ResPQ'

    def __str__(self):
        return '(resPQ (ID: 0x5162463) = (nonce={}, server_nonce={}, pq={}, server_public_key_fingerprints={}))'.format(str(self.nonce), str(self.server_nonce), str(self.pq), None if not self.server_public_key_fingerprints else [str(_) for _ in self.server_public_key_fingerprints])



from telethon.tl.mtproto_request import MTProtoRequest


class PeerNotifyEventsEmpty(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    peerNotifyEventsEmpty#add53cb3  = PeerNotifyEvents"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xadd53cb3

    def __init__(self):
        super().__init__()

    def on_send(self, writer):
        writer.write_int(PeerNotifyEventsEmpty.constructor_id, signed=False)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return PeerNotifyEventsEmpty()

    def on_response(self, reader):
        pass

    def __repr__(self):
        return 'peerNotifyEventsEmpty#add53cb3  = PeerNotifyEvents'

    def __str__(self):
        return '(peerNotifyEventsEmpty (ID: 0xadd53cb3) = ())'.format()



import re


class ReadCancelledError(Exception):
    """Occurs when a read operation was cancelled"""
    def __init__(self):
        super().__init__(self, 'The read operation was cancelled.')


class InvalidParameterError(Exception):
    """Occurs when an invalid parameter is given, for example,
    when either A or B are required but none is given"""


class TypeNotFoundError(Exception):
    """Occurs when a type is not found, for example,
    when trying to read a TLObject with an invalid constructor code"""
    def __init__(self, invalid_constructor_id):
        super().__init__(self, 'Could not find a matching Constructor ID for the TLObject '
                               'that was supposed to be read with ID {}. Most likely, a TLObject '
                               'was trying to be read when it should not be read.'
                               .format(hex(invalid_constructor_id)))

        self.invalid_constructor_id = invalid_constructor_id


class InvalidDCError(Exception):
    def __init__(self, new_dc):
        super().__init__(self,  'Your phone number is registered to #{} DC. '
                                'This should have been handled automatically; '
                                'if it has not, please restart the app.'.format(new_dc))

        self.new_dc = new_dc


class InvalidChecksumError(Exception):
    def __init__(self, checksum, valid_checksum):
        super().__init__(self,  'Invalid checksum ({} when {} was expected). This packet should be skipped.'
                                .format(checksum, valid_checksum))

        self.checksum = checksum
        self.valid_checksum = valid_checksum


class RPCError(Exception):

    CodeMessages = {
        303: ('ERROR_SEE_OTHER', 'The request must be repeated, but directed to a different data center.'),

        400: ('BAD_REQUEST', 'The query contains errors. In the event that a request was created using a '
                             'form and contains user generated data, the user should be notified that the '
                             'data must be corrected before the query is repeated.'),

        401: ('UNAUTHORIZED', 'There was an unauthorized attempt to use functionality available only to '
                              'authorized users.'),

        403: ('FORBIDDEN', 'Privacy violation. For example, an attempt to write a message to someone who '
                           'has blacklisted the current user.'),

        404: ('NOT_FOUND', 'An attempt to invoke a non-existent object, such as a method.'),

        420: ('FLOOD', 'The maximum allowed number of attempts to invoke the given method with '
                       'the given input parameters has been exceeded. For example, in an attempt '
                       'to request a large number of text messages (SMS) for the same phone number.'),

        500: ('INTERNAL', 'An internal server error occurred while a request was being processed; '
                          'for example, there was a disruption while accessing a database or file storage.')
    }

    ErrorMessages = {
        # 303 ERROR_SEE_OTHER
        'FILE_MIGRATE_(\d+)': 'The file to be accessed is currently stored in a different data center (#{}).',

        'PHONE_MIGRATE_(\d+)': 'The phone number a user is trying to use for authorization is associated '
                               'with a different data center (#{}).',

        'NETWORK_MIGRATE_(\d+)': 'The source IP address is associated with a different data center (#{}, '
                                 'for registration).',

        'USER_MIGRATE_(\d+)': 'The user whose identity is being used to execute queries is associated with '
                              'a different data center  (#{} for registration).',

        # 400 BAD_REQUEST
        'FIRSTNAME_INVALID': 'The first name is invalid.',

        'LASTNAME_INVALID': 'The last name is invalid.',

        'PHONE_NUMBER_INVALID': 'The phone number is invalid.',

        'PHONE_CODE_HASH_EMPTY': 'The phone code hash is missing.',

        'PHONE_CODE_EMPTY': 'The phone code is missing.',

        'PHONE_CODE_INVALID': 'The phone code entered was invalid.',

        'PHONE_CODE_EXPIRED': 'The confirmation code has expired.',

        'API_ID_INVALID': 'The api_id/api_hash combination is invalid.',

        'PHONE_NUMBER_OCCUPIED': 'The phone number is already in use.',

        'PHONE_NUMBER_UNOCCUPIED': 'The phone number is not yet being used.',

        'USERS_TOO_FEW': 'Not enough users (to create a chat, for example).',

        'USERS_TOO_MUCH': 'The maximum number of users has been exceeded (to create a chat, for example).',

        'TYPE_CONSTRUCTOR_INVALID': 'The type constructor is invalid.',

        'FILE_PART_INVALID': 'The file part number is invalid.',

        'FILE_PARTS_INVALID': 'The number of file parts is invalid.',

        'FILE_PART_(\d+)_MISSING': 'Part {} of the file is missing from storage.',

        'MD5_CHECKSUM_INVALID': 'The MD5 checksums do not match.',

        'PHOTO_INVALID_DIMENSIONS': 'The photo dimensions are invalid.',

        'FIELD_NAME_INVALID': 'The field with the name FIELD_NAME is invalid.',

        'FIELD_NAME_EMPTY': 'The field with the name FIELD_NAME is missing.',

        'MSG_WAIT_FAILED': 'A waiting call returned an error.',

        'CHAT_ADMIN_REQUIRED': 'Chat admin privileges are required to do that in the specified chat '
                               '(for example, to send a message in a channel which is not yours).',

        # 401 UNAUTHORIZED
        'AUTH_KEY_UNREGISTERED': 'The key is not registered in the system.',

        'AUTH_KEY_INVALID': 'The key is invalid.',

        'USER_DEACTIVATED': 'The user has been deleted/deactivated.',

        'SESSION_REVOKED': 'The authorization has been invalidated, because of the user terminating all sessions.',

        'SESSION_EXPIRED': 'The authorization has expired.',

        'ACTIVE_USER_REQUIRED': 'The method is only available to already activated users.',

        'AUTH_KEY_PERM_EMPTY': 'The method is unavailable for temporary authorization key, not bound to permanent.',

        # 420 FLOOD
        'FLOOD_WAIT_(\d+)': 'A wait of {} seconds is required.'
    }

    def __init__(self, code, message):
        self.code = code
        self.code_meaning = RPCError.CodeMessages[code]

        self.message = message
        self.must_resend = code == 303  # ERROR_SEE_OTHER, "The request must be repeated"

        called_super = False
        for key, error_msg in RPCError.ErrorMessages.items():
            match = re.match(key, message)
            if match:
                # Get additional_data, if any
                if match.groups():
                    self.additional_data = int(match.group(1))
                    super().__init__(self, error_msg.format(self.additional_data))
                else:
                    self.additional_data = None
                    super().__init__(self, error_msg)

                called_super = True
                break

        if not called_super:
            super().__init__(self, 'Unknown error message with code {}: {}'.format(code, message))


class BadMessageError(Exception):
    """Occurs when handling a bad_message_notification"""
    ErrorMessages = {
        16: 'msg_id too low (most likely, client time is wrong it would be worthwhile to '
            'synchronize it using msg_id notifications and re-send the original message '
            'with the "correct" msg_id or wrap it in a container with a new msg_id if the '
            'original message had waited too long on the client to be transmitted).',

        17: 'msg_id too high (similar to the previous case, the client time has to be '
            'synchronized, and the message re-sent with the correct msg_id).',

        18: 'Incorrect two lower order msg_id bits (the server expects client message msg_id '
            'to be divisible by 4).',

        19: 'Container msg_id is the same as msg_id of a previously received message '
            '(this must never happen).',

        20: 'Message too old, and it cannot be verified whether the server has received a '
            'message with this msg_id or not.',

        32: 'msg_seqno too low (the server has already received a message with a lower '
            'msg_id but with either a higher or an equal and odd seqno).',

        33: 'msg_seqno too high (similarly, there is a message with a higher msg_id but with '
            'either a lower or an equal and odd seqno).',

        34: 'An even msg_seqno expected (irrelevant message), but odd received.',

        35: 'Odd msg_seqno expected (relevant message), but even received.',

        48: 'Incorrect server salt (in this case, the bad_server_salt response is received with '
            'the correct salt, and the message is to be re-sent with it).',

        64: 'Invalid container.'
    }

    def __init__(self, code):
        super().__init__(self, BadMessageError
                         .ErrorMessages.get(code,'Unknown error code (this should not happen): {}.'.format(code)))

        self.code = code



from telethon.tl.mtproto_request import MTProtoRequest


class EncryptedFile(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    encryptedFile#4a70994c id:long access_hash:long size:int dc_id:int key_fingerprint:int = EncryptedFile"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x4a70994c

    def __init__(self, id, access_hash, size, dc_id, key_fingerprint):
        """
        :param id: Telegram type: «long».
        :param access_hash: Telegram type: «long».
        :param size: Telegram type: «int».
        :param dc_id: Telegram type: «int».
        :param key_fingerprint: Telegram type: «int».
        """
        super().__init__()

        self.id = id
        self.access_hash = access_hash
        self.size = size
        self.dc_id = dc_id
        self.key_fingerprint = key_fingerprint

    def on_send(self, writer):
        writer.write_int(EncryptedFile.constructor_id, signed=False)
        writer.write_long(self.id)
        writer.write_long(self.access_hash)
        writer.write_int(self.size)
        writer.write_int(self.dc_id)
        writer.write_int(self.key_fingerprint)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return EncryptedFile(None, None, None, None, None)

    def on_response(self, reader):
        self.id = reader.read_long()
        self.access_hash = reader.read_long()
        self.size = reader.read_int()
        self.dc_id = reader.read_int()
        self.key_fingerprint = reader.read_int()

    def __repr__(self):
        return 'encryptedFile#4a70994c id:long access_hash:long size:int dc_id:int key_fingerprint:int = EncryptedFile'

    def __str__(self):
        return '(encryptedFile (ID: 0x4a70994c) = (id={}, access_hash={}, size={}, dc_id={}, key_fingerprint={}))'.format(str(self.id), str(self.access_hash), str(self.size), str(self.dc_id), str(self.key_fingerprint))



from telethon.tl.types.auth.checked_phone import CheckedPhone
from telethon.tl.types.auth.sent_code import SentCode
from telethon.tl.types.auth.authorization import Authorization
from telethon.tl.types.auth.exported_authorization import ExportedAuthorization
from telethon.tl.types.auth.password_recovery import PasswordRecovery
from telethon.tl.types.auth.code_type_sms import CodeTypeSms
from telethon.tl.types.auth.code_type_call import CodeTypeCall
from telethon.tl.types.auth.code_type_flash_call import CodeTypeFlashCall
from telethon.tl.types.auth.sent_code_type_app import SentCodeTypeApp
from telethon.tl.types.auth.sent_code_type_sms import SentCodeTypeSms
from telethon.tl.types.auth.sent_code_type_call import SentCodeTypeCall
from telethon.tl.types.auth.sent_code_type_flash_call import SentCodeTypeFlashCall



from telethon.tl.mtproto_request import MTProtoRequest


class KeyboardButton(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    keyboardButton#a2fa4880 text:string = KeyboardButton"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xa2fa4880

    def __init__(self, text):
        """
        :param text: Telegram type: «string».
        """
        super().__init__()

        self.text = text

    def on_send(self, writer):
        writer.write_int(KeyboardButton.constructor_id, signed=False)
        writer.tgwrite_string(self.text)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return KeyboardButton(None)

    def on_response(self, reader):
        self.text = reader.tgread_string()

    def __repr__(self):
        return 'keyboardButton#a2fa4880 text:string = KeyboardButton'

    def __str__(self):
        return '(keyboardButton (ID: 0xa2fa4880) = (text={}))'.format(str(self.text))



from telethon.tl.mtproto_request import MTProtoRequest


class SetGameScoreRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.setGameScore#8ef8ecc0 flags:None edit_message:flags.0?true peer:InputPeer id:int user_id:InputUser score:int = Updates"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x8ef8ecc0

    def __init__(self, peer, id, user_id, score, edit_message=None):
        """
        :param edit_message: Telegram type: «true».
        :param peer: Telegram type: «InputPeer».
        :param id: Telegram type: «int».
        :param user_id: Telegram type: «InputUser».
        :param score: Telegram type: «int».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.edit_message = edit_message
        self.peer = peer
        self.id = id
        self.user_id = user_id
        self.score = score

    def on_send(self, writer):
        writer.write_int(SetGameScoreRequest.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.edit_message else 0
        writer.write_int(flags)

        self.peer.on_send(writer)
        writer.write_int(self.id)
        self.user_id.on_send(writer)
        writer.write_int(self.score)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return SetGameScoreRequest(None, None, None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.setGameScore#8ef8ecc0 flags:None edit_message:flags.0?true peer:InputPeer id:int user_id:InputUser score:int = Updates'

    def __str__(self):
        return '(messages.setGameScore (ID: 0x8ef8ecc0) = (edit_message={}, peer={}, id={}, user_id={}, score={}))'.format(str(self.edit_message), str(self.peer), str(self.id), str(self.user_id), str(self.score))



from telethon.tl.functions.contacts.get_statuses import GetStatusesRequest
from telethon.tl.functions.contacts.get_contacts import GetContactsRequest
from telethon.tl.functions.contacts.import_contacts import ImportContactsRequest
from telethon.tl.functions.contacts.delete_contact import DeleteContactRequest
from telethon.tl.functions.contacts.delete_contacts import DeleteContactsRequest
from telethon.tl.functions.contacts.block import BlockRequest
from telethon.tl.functions.contacts.unblock import UnblockRequest
from telethon.tl.functions.contacts.get_blocked import GetBlockedRequest
from telethon.tl.functions.contacts.export_card import ExportCardRequest
from telethon.tl.functions.contacts.import_card import ImportCardRequest
from telethon.tl.functions.contacts.search import SearchRequest
from telethon.tl.functions.contacts.resolve_username import ResolveUsernameRequest
from telethon.tl.functions.contacts.get_top_peers import GetTopPeersRequest
from telethon.tl.functions.contacts.reset_top_peer_rating import ResetTopPeerRatingRequest



import os
import time
import telethon.helpers as utils
from telethon.utils import BinaryWriter, BinaryReader
from telethon.crypto import AES, AuthKey, Factorizator, RSA
from telethon.network import MtProtoPlainSender


def do_authentication(transport):
    """Executes the authentication process with the Telegram servers.
    If no error is rose, returns both the authorization key and the time offset"""
    sender = MtProtoPlainSender(transport)

    # Step 1 sending: PQ Request
    nonce = os.urandom(16)
    with BinaryWriter() as writer:
        writer.write_int(0x60469778, signed=False)  # Constructor number
        writer.write(nonce)
        sender.send(writer.get_bytes())

    # Step 1 response: PQ Request
    pq, pq_bytes, server_nonce, fingerprints = None, None, None, []
    with BinaryReader(sender.receive()) as reader:
        response_code = reader.read_int(signed=False)
        if response_code != 0x05162463:
            raise AssertionError('Invalid response code: {}'.format(hex(response_code)))

        nonce_from_server = reader.read(16)
        if nonce_from_server != nonce:
            raise AssertionError('Invalid nonce from server')

        server_nonce = reader.read(16)

        pq_bytes = reader.tgread_bytes()
        pq = get_int(pq_bytes)

        vector_id = reader.read_int()
        if vector_id != 0x1cb5c415:
            raise AssertionError('Invalid vector constructor ID: {}'.format(hex(response_code)))

        fingerprints = []
        fingerprint_count = reader.read_int()
        for _ in range(fingerprint_count):
            fingerprints.append(reader.read(8))

    # Step 2 sending: DH Exchange
    new_nonce = os.urandom(32)
    p, q = Factorizator.factorize(pq)
    with BinaryWriter() as pq_inner_data_writer:
        pq_inner_data_writer.write_int(0x83c95aec, signed=False)  # PQ Inner Data
        pq_inner_data_writer.tgwrite_bytes(get_byte_array(pq, signed=False))
        pq_inner_data_writer.tgwrite_bytes(get_byte_array(min(p, q), signed=False))
        pq_inner_data_writer.tgwrite_bytes(get_byte_array(max(p, q), signed=False))
        pq_inner_data_writer.write(nonce)
        pq_inner_data_writer.write(server_nonce)
        pq_inner_data_writer.write(new_nonce)

        cipher_text, target_fingerprint = None, None
        for fingerprint in fingerprints:
            cipher_text = RSA.encrypt(get_fingerprint_text(fingerprint), pq_inner_data_writer.get_bytes())

            if cipher_text is not None:
                target_fingerprint = fingerprint
                break

        if cipher_text is None:
            raise AssertionError('Could not find a valid key for fingerprints: {}'
                                 .format(', '.join([get_fingerprint_text(f) for f in fingerprints])))

        with BinaryWriter() as req_dh_params_writer:
            req_dh_params_writer.write_int(0xd712e4be, signed=False)  # Req DH Params
            req_dh_params_writer.write(nonce)
            req_dh_params_writer.write(server_nonce)
            req_dh_params_writer.tgwrite_bytes(get_byte_array(min(p, q), signed=False))
            req_dh_params_writer.tgwrite_bytes(get_byte_array(max(p, q), signed=False))
            req_dh_params_writer.write(target_fingerprint)
            req_dh_params_writer.tgwrite_bytes(cipher_text)

            req_dh_params_bytes = req_dh_params_writer.get_bytes()
            sender.send(req_dh_params_bytes)

    # Step 2 response: DH Exchange
    encrypted_answer = None
    with BinaryReader(sender.receive()) as reader:
        response_code = reader.read_int(signed=False)

        if response_code == 0x79cb045d:
            raise AssertionError('Server DH params fail: TODO')

        if response_code != 0xd0e8075c:
            raise AssertionError('Invalid response code: {}'.format(hex(response_code)))

        nonce_from_server = reader.read(16)
        if nonce_from_server != nonce:
            raise NotImplementedError('Invalid nonce from server')

        server_nonce_from_server = reader.read(16)
        if server_nonce_from_server != server_nonce:
            raise NotImplementedError('Invalid server nonce from server')

        encrypted_answer = reader.tgread_bytes()

    # Step 3 sending: Complete DH Exchange
    key, iv = utils.generate_key_data_from_nonces(server_nonce, new_nonce)
    plain_text_answer = AES.decrypt_ige(encrypted_answer, key, iv)

    g, dh_prime, ga, time_offset = None, None, None, None
    with BinaryReader(plain_text_answer) as dh_inner_data_reader:
        hashsum = dh_inner_data_reader.read(20)
        code = dh_inner_data_reader.read_int(signed=False)
        if code != 0xb5890dba:
            raise AssertionError('Invalid DH Inner Data code: {}'.format(code))

        nonce_from_server1 = dh_inner_data_reader.read(16)
        if nonce_from_server1 != nonce:
            raise AssertionError('Invalid nonce in encrypted answer')

        server_nonce_from_server1 = dh_inner_data_reader.read(16)
        if server_nonce_from_server1 != server_nonce:
            raise AssertionError('Invalid server nonce in encrypted answer')

        g = dh_inner_data_reader.read_int()
        dh_prime = get_int(dh_inner_data_reader.tgread_bytes(), signed=False)
        ga = get_int(dh_inner_data_reader.tgread_bytes(), signed=False)

        server_time = dh_inner_data_reader.read_int()
        time_offset = server_time - int(time.time())

    b = get_int(os.urandom(2048), signed=False)
    gb = pow(g, b, dh_prime)
    gab = pow(ga, b, dh_prime)

    # Prepare client DH Inner Data
    with BinaryWriter() as client_dh_inner_data_writer:
        client_dh_inner_data_writer.write_int(0x6643b654, signed=False)  # Client DH Inner Data
        client_dh_inner_data_writer.write(nonce)
        client_dh_inner_data_writer.write(server_nonce)
        client_dh_inner_data_writer.write_long(0)  # TODO retry_id
        client_dh_inner_data_writer.tgwrite_bytes(get_byte_array(gb, signed=False))

        with BinaryWriter() as client_dh_inner_data_with_hash_writer:
            client_dh_inner_data_with_hash_writer.write(utils.sha1(client_dh_inner_data_writer.get_bytes()))
            client_dh_inner_data_with_hash_writer.write(client_dh_inner_data_writer.get_bytes())
            client_dh_inner_data_bytes = client_dh_inner_data_with_hash_writer.get_bytes()

    # Encryption
    client_dh_inner_data_encrypted_bytes = AES.encrypt_ige(client_dh_inner_data_bytes, key, iv)

    # Prepare Set client DH params
    with BinaryWriter() as set_client_dh_params_writer:
        set_client_dh_params_writer.write_int(0xf5045f1f, signed=False)
        set_client_dh_params_writer.write(nonce)
        set_client_dh_params_writer.write(server_nonce)
        set_client_dh_params_writer.tgwrite_bytes(client_dh_inner_data_encrypted_bytes)

        set_client_dh_params_bytes = set_client_dh_params_writer.get_bytes()
        sender.send(set_client_dh_params_bytes)

    # Step 3 response: Complete DH Exchange
    with BinaryReader(sender.receive()) as reader:
        code = reader.read_int(signed=False)
        if code == 0x3bcbf734:  # DH Gen OK
            nonce_from_server = reader.read(16)
            if nonce_from_server != nonce:
                raise NotImplementedError('Invalid nonce from server')

            server_nonce_from_server = reader.read(16)
            if server_nonce_from_server != server_nonce:
                raise NotImplementedError('Invalid server nonce from server')

            new_nonce_hash1 = reader.read(16)
            auth_key = AuthKey(get_byte_array(gab, signed=False))

            new_nonce_hash_calculated = auth_key.calc_new_nonce_hash(new_nonce, 1)
            if new_nonce_hash1 != new_nonce_hash_calculated:
                raise AssertionError('Invalid new nonce hash')

            return auth_key, time_offset

        elif code == 0x46dc1fb9:  # DH Gen Retry
            raise NotImplementedError('dh_gen_retry')

        elif code == 0xa69dae02:  # DH Gen Fail
            raise NotImplementedError('dh_gen_fail')

        else:
            raise AssertionError('DH Gen unknown: {}'.format(hex(code)))


def get_fingerprint_text(fingerprint):
    """Gets a fingerprint text in 01-23-45-67-89-AB-CD-EF format (no hyphens)"""
    return ''.join(hex(b)[2:].rjust(2, '0').upper() for b in fingerprint)


# The following methods operate in big endian (unlike most of Telegram API) because:
# > "...pq is a representation of a natural number (in binary *big endian* format)..."
# > "...current value of dh_prime equals (in *big-endian* byte order)..."
# Reference: https://core.telegram.org/mtproto/auth_key
def get_byte_array(integer, signed):
    """Gets the arbitrary-length byte array corresponding to the given integer"""
    bits = integer.bit_length()
    byte_length = (bits + 8 - 1) // 8  # 8 bits per byte
    return int.to_bytes(integer, length=byte_length, byteorder='big', signed=signed)


def get_int(byte_array, signed=True):
    """Gets the specified integer from its byte array. This should be used by the authenticator,
       who requires the data to be in big endian"""
    return int.from_bytes(byte_array, byteorder='big', signed=signed)



from telethon.tl.mtproto_request import MTProtoRequest


class PhotoSizeEmpty(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    photoSizeEmpty#0e17e23c type:string = PhotoSize"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xe17e23c

    def __init__(self, type):
        """
        :param type: Telegram type: «string».
        """
        super().__init__()

        self.type = type

    def on_send(self, writer):
        writer.write_int(PhotoSizeEmpty.constructor_id, signed=False)
        writer.tgwrite_string(self.type)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return PhotoSizeEmpty(None)

    def on_response(self, reader):
        self.type = reader.tgread_string()

    def __repr__(self):
        return 'photoSizeEmpty#0e17e23c type:string = PhotoSize'

    def __str__(self):
        return '(photoSizeEmpty (ID: 0xe17e23c) = (type={}))'.format(str(self.type))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateNewEncryptedMessage(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateNewEncryptedMessage#12bcbd9a message:EncryptedMessage qts:int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x12bcbd9a

    def __init__(self, message, qts):
        """
        :param message: Telegram type: «EncryptedMessage».
        :param qts: Telegram type: «int».
        """
        super().__init__()

        self.message = message
        self.qts = qts

    def on_send(self, writer):
        writer.write_int(UpdateNewEncryptedMessage.constructor_id, signed=False)
        self.message.on_send(writer)
        writer.write_int(self.qts)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateNewEncryptedMessage(None, None)

    def on_response(self, reader):
        self.message = reader.tgread_object()
        self.qts = reader.read_int()

    def __repr__(self):
        return 'updateNewEncryptedMessage#12bcbd9a message:EncryptedMessage qts:int = Update'

    def __str__(self):
        return '(updateNewEncryptedMessage (ID: 0x12bcbd9a) = (message={}, qts={}))'.format(str(self.message), str(self.qts))



from telethon.tl.mtproto_request import MTProtoRequest


class FoundGif(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    foundGif#162ecc1f url:string thumb_url:string content_url:string content_type:string w:int h:int = FoundGif"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x162ecc1f

    def __init__(self, url, thumb_url, content_url, content_type, w, h):
        """
        :param url: Telegram type: «string».
        :param thumb_url: Telegram type: «string».
        :param content_url: Telegram type: «string».
        :param content_type: Telegram type: «string».
        :param w: Telegram type: «int».
        :param h: Telegram type: «int».
        """
        super().__init__()

        self.url = url
        self.thumb_url = thumb_url
        self.content_url = content_url
        self.content_type = content_type
        self.w = w
        self.h = h

    def on_send(self, writer):
        writer.write_int(FoundGif.constructor_id, signed=False)
        writer.tgwrite_string(self.url)
        writer.tgwrite_string(self.thumb_url)
        writer.tgwrite_string(self.content_url)
        writer.tgwrite_string(self.content_type)
        writer.write_int(self.w)
        writer.write_int(self.h)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return FoundGif(None, None, None, None, None, None)

    def on_response(self, reader):
        self.url = reader.tgread_string()
        self.thumb_url = reader.tgread_string()
        self.content_url = reader.tgread_string()
        self.content_type = reader.tgread_string()
        self.w = reader.read_int()
        self.h = reader.read_int()

    def __repr__(self):
        return 'foundGif#162ecc1f url:string thumb_url:string content_url:string content_type:string w:int h:int = FoundGif'

    def __str__(self):
        return '(foundGif (ID: 0x162ecc1f) = (url={}, thumb_url={}, content_url={}, content_type={}, w={}, h={}))'.format(str(self.url), str(self.thumb_url), str(self.content_url), str(self.content_type), str(self.w), str(self.h))



from binascii import crc32
from datetime import timedelta

from telethon.network import TcpClient
from telethon.errors import *
from telethon.utils import BinaryWriter


class TcpTransport:
    def __init__(self, ip_address, port):
        self.tcp_client = TcpClient()
        self.send_counter = 0

        self.tcp_client.connect(ip_address, port)

    # Original reference: https://core.telegram.org/mtproto#tcp-transport
    # The packets are encoded as: total length, sequence number, packet and checksum (CRC32)
    def send(self, packet):
        """Sends the given packet (bytes array) to the connected peer"""
        if not self.tcp_client.connected:
            raise ConnectionError('Client not connected to server.')

        with BinaryWriter() as writer:
            writer.write_int(len(packet) + 12)  # 12 = size_of (integer) * 3
            writer.write_int(self.send_counter)
            writer.write(packet)

            crc = crc32(writer.get_bytes())
            writer.write_int(crc, signed=False)
            
            self.send_counter += 1
            self.tcp_client.write(writer.get_bytes())

    def receive(self, timeout=timedelta(seconds=5)):
        """Receives a TCP message (tuple(sequence number, body)) from the connected peer.
           There is a default timeout of 5 seconds before the operation is cancelled.
           Timeout can be set to None for no timeout"""

        # First read everything we need
        packet_length_bytes = self.tcp_client.read(4, timeout)
        packet_length = int.from_bytes(packet_length_bytes, byteorder='little')

        seq_bytes = self.tcp_client.read(4, timeout)
        seq = int.from_bytes(seq_bytes, byteorder='little')

        body = self.tcp_client.read(packet_length - 12, timeout)

        checksum = int.from_bytes(self.tcp_client.read(4, timeout),
                                  byteorder='little',
                                  signed=False)

        # Then perform the checks
        rv = packet_length_bytes + seq_bytes + body
        valid_checksum = crc32(rv)

        if checksum != valid_checksum:
            raise InvalidChecksumError(checksum, valid_checksum)

        # If we passed the tests, we can then return a valid TCP message
        return seq, body

    def close(self):
        if self.tcp_client.connected:
            self.tcp_client.close()

    def cancel_receive(self):
        """Cancels (stops) trying to receive from the
        remote peer and raises a ReadCancelledError"""
        self.tcp_client.cancel_read()

    def get_client_delay(self):
        """Gets the client read delay"""
        return self.tcp_client.delay



from telethon.tl.mtproto_request import MTProtoRequest


class GetPrivacyRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    account.getPrivacy#dadbc950 key:InputPrivacyKey = account.PrivacyRules"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xdadbc950

    def __init__(self, key):
        """
        :param key: Telegram type: «InputPrivacyKey».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.key = key

    def on_send(self, writer):
        writer.write_int(GetPrivacyRequest.constructor_id, signed=False)
        self.key.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return GetPrivacyRequest(None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'account.getPrivacy#dadbc950 key:InputPrivacyKey = account.PrivacyRules'

    def __str__(self):
        return '(account.getPrivacy (ID: 0xdadbc950) = (key={}))'.format(str(self.key))



from telethon.tl.mtproto_request import MTProtoRequest


class NearestDc(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    nearestDc#8e1a1775 country:string this_dc:int nearest_dc:int = NearestDc"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x8e1a1775

    def __init__(self, country, this_dc, nearest_dc):
        """
        :param country: Telegram type: «string».
        :param this_dc: Telegram type: «int».
        :param nearest_dc: Telegram type: «int».
        """
        super().__init__()

        self.country = country
        self.this_dc = this_dc
        self.nearest_dc = nearest_dc

    def on_send(self, writer):
        writer.write_int(NearestDc.constructor_id, signed=False)
        writer.tgwrite_string(self.country)
        writer.write_int(self.this_dc)
        writer.write_int(self.nearest_dc)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return NearestDc(None, None, None)

    def on_response(self, reader):
        self.country = reader.tgread_string()
        self.this_dc = reader.read_int()
        self.nearest_dc = reader.read_int()

    def __repr__(self):
        return 'nearestDc#8e1a1775 country:string this_dc:int nearest_dc:int = NearestDc'

    def __str__(self):
        return '(nearestDc (ID: 0x8e1a1775) = (country={}, this_dc={}, nearest_dc={}))'.format(str(self.country), str(self.this_dc), str(self.nearest_dc))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateUserTyping(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateUserTyping#5c486927 user_id:int action:SendMessageAction = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x5c486927

    def __init__(self, user_id, action):
        """
        :param user_id: Telegram type: «int».
        :param action: Telegram type: «SendMessageAction».
        """
        super().__init__()

        self.user_id = user_id
        self.action = action

    def on_send(self, writer):
        writer.write_int(UpdateUserTyping.constructor_id, signed=False)
        writer.write_int(self.user_id)
        self.action.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateUserTyping(None, None)

    def on_response(self, reader):
        self.user_id = reader.read_int()
        self.action = reader.tgread_object()

    def __repr__(self):
        return 'updateUserTyping#5c486927 user_id:int action:SendMessageAction = Update'

    def __str__(self):
        return '(updateUserTyping (ID: 0x5c486927) = (user_id={}, action={}))'.format(str(self.user_id), str(self.action))



from telethon.tl.mtproto_request import MTProtoRequest


class PrivacyRules(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    account.privacyRules#554abb6f rules:Vector<PrivacyRule> users:Vector<User> = account.PrivacyRules"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x554abb6f

    def __init__(self, rules, users):
        """
        :param rules: Telegram type: «PrivacyRule». Must be a list.
        :param users: Telegram type: «User». Must be a list.
        """
        super().__init__()

        self.rules = rules
        self.users = users

    def on_send(self, writer):
        writer.write_int(PrivacyRules.constructor_id, signed=False)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.rules))
        for rules_item in self.rules:
            rules_item.on_send(writer)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.users))
        for users_item in self.users:
            users_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return PrivacyRules(None, None)

    def on_response(self, reader):
        reader.read_int()  # Vector's constructor ID
        self.rules = []  # Initialize an empty list
        rules_len = reader.read_int()
        for _ in range(rules_len):
            rules_item = reader.tgread_object()
            self.rules.append(rules_item)

        reader.read_int()  # Vector's constructor ID
        self.users = []  # Initialize an empty list
        users_len = reader.read_int()
        for _ in range(users_len):
            users_item = reader.tgread_object()
            self.users.append(users_item)

    def __repr__(self):
        return 'account.privacyRules#554abb6f rules:Vector<PrivacyRule> users:Vector<User> = account.PrivacyRules'

    def __str__(self):
        return '(account.privacyRules (ID: 0x554abb6f) = (rules={}, users={}))'.format(None if not self.rules else [str(_) for _ in self.rules], None if not self.users else [str(_) for _ in self.users])



from telethon.tl.mtproto_request import MTProtoRequest


class ExportAuthorizationRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    auth.exportAuthorization#e5bfffcd dc_id:int = auth.ExportedAuthorization"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xe5bfffcd

    def __init__(self, dc_id):
        """
        :param dc_id: Telegram type: «int».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.dc_id = dc_id

    def on_send(self, writer):
        writer.write_int(ExportAuthorizationRequest.constructor_id, signed=False)
        writer.write_int(self.dc_id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ExportAuthorizationRequest(None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'auth.exportAuthorization#e5bfffcd dc_id:int = auth.ExportedAuthorization'

    def __str__(self):
        return '(auth.exportAuthorization (ID: 0xe5bfffcd) = (dc_id={}))'.format(str(self.dc_id))



from telethon.tl.mtproto_request import MTProtoRequest


class SentEncryptedFile(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.sentEncryptedFile#9493ff32 date:date file:EncryptedFile = messages.SentEncryptedMessage"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x9493ff32

    def __init__(self, date, file):
        """
        :param date: Telegram type: «date».
        :param file: Telegram type: «EncryptedFile».
        """
        super().__init__()

        self.date = date
        self.file = file

    def on_send(self, writer):
        writer.write_int(SentEncryptedFile.constructor_id, signed=False)
        writer.tgwrite_date(self.date)
        self.file.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return SentEncryptedFile(None, None)

    def on_response(self, reader):
        self.date = reader.tgread_date()
        self.file = reader.tgread_object()

    def __repr__(self):
        return 'messages.sentEncryptedFile#9493ff32 date:date file:EncryptedFile = messages.SentEncryptedMessage'

    def __str__(self):
        return '(messages.sentEncryptedFile (ID: 0x9493ff32) = (date={}, file={}))'.format(str(self.date), str(self.file))



import platform
from datetime import datetime, timedelta
from hashlib import md5
from os import path, listdir
from mimetypes import guess_type

# For sending and receiving requests
from telethon.tl import MTProtoRequest
from telethon.tl import Session

# The Requests and types that we'll be using
from telethon.tl.functions.upload import SaveBigFilePartRequest
from telethon.tl.functions import InvokeWithLayerRequest, InitConnectionRequest
from telethon.tl.functions.help import GetConfigRequest
from telethon.tl.functions.auth import SendCodeRequest, SignInRequest, SignUpRequest, LogOutRequest
from telethon.tl.functions.upload import SaveFilePartRequest, GetFileRequest
from telethon.tl.functions.messages import \
    GetDialogsRequest, GetHistoryRequest, \
    SendMessageRequest, SendMediaRequest, \
    ReadHistoryRequest

# All the types we need to work with
from telethon.tl.types import \
    InputPeerEmpty, \
    UserProfilePhotoEmpty, ChatPhotoEmpty, \
    InputFile, InputFileLocation, InputMediaUploadedPhoto, InputMediaUploadedDocument, \
    MessageMediaContact, MessageMediaDocument, MessageMediaPhoto, \
    DocumentAttributeAudio, DocumentAttributeFilename, InputDocumentFileLocation

# Import some externalized utilities to work with the Telegram types and more
import telethon.helpers as utils
import telethon.network.authenticator as authenticator
from telethon.utils import find_user_or_chat, get_appropiate_part_size, get_extension

from telethon.errors import *
from telethon.network import MtProtoSender, TcpTransport
from telethon.parser.markdown_parser import parse_message_entities
from telethon.tl.all_tlobjects import layer


class TelegramClient:

    # Current TelegramClient version
    __version__ = '0.6'

    # region Initialization

    def __init__(self, session_user_id, api_id, api_hash):
        if api_id is None or api_hash is None:
            raise PermissionError('Your API ID or Hash are invalid. Please read "Requirements" on README.rst')

        self.api_id = api_id
        self.api_hash = api_hash

        self.session = Session.try_load_or_create_new(session_user_id)
        self.transport = TcpTransport(self.session.server_address, self.session.port)

        # These will be set later
        self.dc_options = None
        self.sender = None
        self.phone_code_hashes = {}

        # We need to be signed in before we can listen for updates
        self.signed_in = False

    # endregion

    # region Connecting

    def connect(self, reconnect=False):
        """Connects to the Telegram servers, executing authentication if required.
           Note that authenticating to the Telegram servers is not the same as authenticating
           the app, which requires to send a code first."""
        try:
            if not self.session.auth_key or reconnect:
                self.session.auth_key, self.session.time_offset = \
                    authenticator.do_authentication(self.transport)

                self.session.save()

            self.sender = MtProtoSender(self.transport, self.session)

            # Now it's time to send an InitConnectionRequest
            # This must always be invoked with the layer we'll be using
            query = InitConnectionRequest(api_id=self.api_id,
                                          device_model=platform.node(),
                                          system_version=platform.system(),
                                          app_version=self.__version__,
                                          lang_code='en',
                                          query=GetConfigRequest())

            result = self.invoke(InvokeWithLayerRequest(layer=layer, query=query))

            # We're only interested in the DC options,
            # although many other options are available!
            self.dc_options = result.dc_options

            # We can now enable these (for such methods such as logout)
            self.sender.ack_requests_confirm = True

            # We're signed in if we're authorized
            self.signed_in = self.is_user_authorized()
            return True
        except RPCError as error:
            print('Could not stabilise initial connection: {}'.format(error))
            return False

    def reconnect_to_dc(self, dc_id):
        """Reconnects to the specified DC ID. This is automatically called after an InvalidDCError is raised"""
        if self.dc_options is None or not self.dc_options:
            raise ConnectionError("Can't reconnect. Stabilise an initial connection first.")

        dc = next(dc for dc in self.dc_options if dc.id == dc_id)

        self.transport.close()
        self.transport = TcpTransport(dc.ip_address, dc.port)
        self.session.server_address = dc.ip_address
        self.session.port = dc.port
        self.session.save()

        self.connect(reconnect=True)

    def disconnect(self):
        """Disconnects from the Telegram server **and pauses all the spawned threads**"""
        if self.sender:
            self.sender.disconnect()

    # endregion

    # region Telegram requests functions

    def invoke(self, request, timeout=timedelta(seconds=5)):
        """Invokes a MTProtoRequest (sends and receives it) and returns its result.
           An optional timeout can be given to cancel the operation after the time delta.
           Timeout can be set to None for no timeout"""
        if not issubclass(type(request), MTProtoRequest):
            raise ValueError('You can only invoke MtProtoRequests')

        self.sender.send(request)
        self.sender.receive(request, timeout)

        return request.result

    # region Authorization requests

    def is_user_authorized(self):
        """Has the user been authorized yet (code request sent and confirmed)?
           Note that this will NOT yield the correct result if the session was revoked by another client!"""
        return self.session.user is not None

    def send_code_request(self, phone_number):
        """Sends a code request to the specified phone number"""
        request = SendCodeRequest(phone_number, self.api_id, self.api_hash)
        completed = False
        while not completed:
            try:
                result = self.invoke(request)
                self.phone_code_hashes[phone_number] = result.phone_code_hash
                completed = True

            except InvalidDCError as error:
                self.reconnect_to_dc(error.new_dc)

    def sign_in(self, phone_number, code):
        """Completes the authorization of a phone number by providing the received code"""
        if phone_number not in self.phone_code_hashes:
            raise ValueError('Please make sure you have called send_code_request first.')

        try:
            result = self.invoke(SignInRequest(
                phone_number, self.phone_code_hashes[phone_number], code))

        except RPCError as error:
            if error.message.startswith('PHONE_CODE_'):
                print(error)
                return False
            else:
                raise error

        # Result is an Auth.Authorization TLObject
        self.session.user = result.user
        self.session.save()

        # Now that we're authorized, set the signed_in flag
        # to True so update handlers can be added
        self.signed_in = True
        return True

    def sign_up(self, phone_number, code, first_name, last_name=''):
        """Signs up to Telegram. Make sure you sent a code request first!"""
        result = self.invoke(SignUpRequest(phone_number=phone_number,
                                           phone_code_hash=self.phone_code_hashes[phone_number],
                                           phone_code=code,
                                           first_name=first_name,
                                           last_name=last_name))

        self.session.user = result.user
        self.session.save()

    def log_out(self):
        """Logs out and deletes the current session. Returns True if everything went OK"""
        try:
            self.invoke(LogOutRequest())
            if not self.session.delete():
                return False

            self.session = None
        except:
            return False

    @staticmethod
    def list_sessions():
        """Lists all the sessions of the users who have ever connected
           using this client and never logged out"""
        return [path.splitext(path.basename(f))[0]  # splitext = split ext (not spli text!)
                for f in listdir('.') if f.endswith('.session')]

    # endregion

    # region Dialogs ("chats") requests

    def get_dialogs(self, count=10, offset_date=None, offset_id=0, offset_peer=InputPeerEmpty()):
        """Returns a tuple of lists ([dialogs], [entities]) with 'count' items each.
           The `entity` represents the user, chat or channel corresponding to that dialog"""

        r = self.invoke(GetDialogsRequest(offset_date=offset_date,
                                          offset_id=offset_id,
                                          offset_peer=offset_peer,
                                          limit=count))
        return (r.dialogs,
                [find_user_or_chat(d.peer, r.users, r.chats) for d in r.dialogs])

    # endregion

    # region Message requests

    def send_message(self, input_peer, message, markdown=False, no_web_page=False):
        """Sends a message to the given input peer and returns the sent message ID"""
        if markdown:
            msg, entities = parse_message_entities(message)
        else:
            msg, entities = message, []

        msg_id = utils.generate_random_long()
        self.invoke(SendMessageRequest(peer=input_peer,
                                       message=msg,
                                       random_id=msg_id,
                                       entities=entities,
                                       no_webpage=no_web_page))
        return msg_id

    def get_message_history(self, input_peer, limit=20,
                            offset_date=None, offset_id=0, max_id=0, min_id=0, add_offset=0):
        """
        Gets the message history for the specified InputPeer

        :param input_peer:  The InputPeer from whom to retrieve the message history
        :param limit:       Number of messages to be retrieved
        :param offset_date: Offset date (messages *previous* to this date will be retrieved)
        :param offset_id:   Offset message ID (only messages *previous* to the given ID will be retrieved)
        :param max_id:      All the messages with a higher (newer) ID or equal to this will be excluded
        :param min_id:      All the messages with a lower (older) ID or equal to this will be excluded
        :param add_offset:  Additional message offset (all of the specified offsets + this offset = older messages)

        :return: A tuple containing total message count and two more lists ([messages], [senders]).
                 Note that the sender can be null if it was not found!
        """
        result = self.invoke(GetHistoryRequest(input_peer,
                                               limit=limit,
                                               offset_date=offset_date,
                                               offset_id=offset_id,
                                               max_id=max_id,
                                               min_id=min_id,
                                               add_offset=add_offset))

        # The result may be a messages slice (not all messages were retrieved) or
        # simply a messages TLObject. In the later case, no "count" attribute is specified:
        # the total messages count is retrieved by counting all the retrieved messages
        total_messages = getattr(result, 'count', len(result.messages))

        # Iterate over all the messages and find the sender User
        users = []
        for msg in result.messages:
            for usr in result.users:
                if msg.from_id == usr.id:
                    users.append(usr)
                    break

        return total_messages, result.messages, users

    def send_read_acknowledge(self, input_peer, messages=None, max_id=None):
        """Sends a "read acknowledge" (i.e., notifying the given peer that we've
           read their messages, also known as the "double check ✅✅").

           Either a list of messages (or a single message) can be given,
           or the maximum message ID (until which message we want to send the read acknowledge).

           Returns an AffectedMessages TLObject"""
        if max_id is None:
            if not messages:
                raise InvalidParameterError('Either a message list or a max_id must be provided.')

            if isinstance(messages, list):
                max_id = max(msg.id for msg in messages)
            else:
                max_id = messages.id

        return self.invoke(ReadHistoryRequest(peer=input_peer, max_id=max_id))

    # endregion

    # TODO Handle media downloading/uploading in a different session?
    #  "It is recommended that large queries (upload.getFile, upload.saveFilePart)
    #   be handled through a separate session and a separate connection"
    # region Uploading media requests

    def upload_file(self, file_path, part_size_kb=None, file_name=None, progress_callback=None):
        """Uploads the specified file_path and returns a handle which can be later used

        :param file_path: The file path of the file that will be uploaded
        :param part_size_kb: The part size when uploading the file. None = Automatic
        :param file_name: The name of the uploaded file. None = Automatic
        :param progress_callback: A callback function which takes two parameters,
                                  uploaded size (in bytes) and total file size (in bytes)
                                  This is called every time a part is uploaded
        """
        file_size = path.getsize(file_path)
        if not part_size_kb:
            part_size_kb = get_appropiate_part_size(file_size)

        if part_size_kb > 512:
            raise ValueError('The part size must be less or equal to 512KB')

        part_size = int(part_size_kb * 1024)
        if part_size % 1024 != 0:
            raise ValueError('The part size must be evenly divisible by 1024')

        # Determine whether the file is too big (over 10MB) or not
        # Telegram does make a distinction between smaller or larger files
        is_large = file_size > 10 * 1024 * 1024
        part_count = (file_size + part_size - 1) // part_size

        # Multiply the datetime timestamp by 10^6 to get the ticks
        # This is high likely going to be unique
        file_id = int(datetime.now().timestamp() * (10 ** 6))
        hash_md5 = md5()

        with open(file_path, 'rb') as file:
            for part_index in range(part_count):
                # Read the file by in chunks of size part_size
                part = file.read(part_size)

                # The SavePartRequest is different depending on whether
                # the file is too large or not (over or less than 10MB)
                if is_large:
                    request = SaveBigFilePartRequest(file_id, part_index, part_count, part)
                else:
                    request = SaveFilePartRequest(file_id, part_index, part)

                # Invoke the file upload and increment both the part index and MD5 checksum
                result = self.invoke(request)
                if result:
                    hash_md5.update(part)
                    if progress_callback:
                        progress_callback(file.tell(), file_size)
                else:
                    raise ValueError('Could not upload file part #{}'.format(part_index))

        # Set a default file name if None was specified
        if not file_name:
            file_name = path.basename(file_path)

        # After the file has been uploaded, we can return a handle pointing to it
        return InputFile(id=file_id,
                         parts=part_count,
                         name=file_name,
                         md5_checksum=hash_md5.hexdigest())

    def send_photo_file(self, input_file, input_peer, caption=''):
        """Sends a previously uploaded input_file
           (which should be a photo) to an input_peer"""
        self.send_media_file(
            InputMediaUploadedPhoto(input_file, caption), input_peer)

    def send_document_file(self, input_file, input_peer, caption=''):
        """Sends a previously uploaded input_file
           (which should be a document) to an input_peer"""

        # Determine mime-type and attributes
        # Take the first element by using [0] since it returns a tuple
        mime_type = guess_type(input_file.name)[0]
        attributes = [
            DocumentAttributeFilename(input_file.name)
            # TODO If the input file is an audio, find out:
            # Performer and song title and add DocumentAttributeAudio
        ]
        # Ensure we have a mime type, any; but it cannot be None
        # «The "octet-stream" subtype is used to indicate that a body contains arbitrary binary data.»
        if not mime_type:
            mime_type = 'application/octet-stream'
        self.send_media_file(InputMediaUploadedDocument(file=input_file,
                                                        mime_type=mime_type,
                                                        attributes=attributes,
                                                        caption=caption), input_peer)

    def send_media_file(self, input_media, input_peer):
        """Sends any input_media (contact, document, photo...) to an input_peer"""
        self.invoke(SendMediaRequest(peer=input_peer,
                                     media=input_media,
                                     random_id=utils.generate_random_long()))

    # endregion

    # region Downloading media requests

    def download_profile_photo(self, profile_photo, file_path,
                               add_extension=True, download_big=True):
        """Downloads the profile photo for an user or a chat (including channels).
           Returns False if no photo was providen, or if it was Empty"""

        if (not profile_photo or
            isinstance(profile_photo, UserProfilePhotoEmpty) or
                isinstance(profile_photo, ChatPhotoEmpty)):
            return False

        if add_extension:
            file_path += get_extension(profile_photo)

        if download_big:
            photo_location = profile_photo.photo_big
        else:
            photo_location = profile_photo.photo_small

        # Download the media with the largest size input file location
        self.download_file_loc(InputFileLocation(volume_id=photo_location.volume_id,
                                                 local_id=photo_location.local_id,
                                                 secret=photo_location.secret),
                               file_path)
        return True

    def download_msg_media(self, message_media, file_path, add_extension=True, progress_callback=None):
        """Downloads the given MessageMedia (Photo, Document or Contact)
           into the desired file_path, optionally finding its extension automatically
           The progress_callback should be a callback function which takes two parameters,
           uploaded size (in bytes) and total file size (in bytes).
           This will be called every time a part is downloaded"""
        if type(message_media) == MessageMediaPhoto:
            return self.download_photo(message_media, file_path, add_extension, progress_callback)

        elif type(message_media) == MessageMediaDocument:
            return self.download_document(message_media, file_path, add_extension, progress_callback)

        elif type(message_media) == MessageMediaContact:
            return self.download_contact(message_media, file_path, add_extension)

    def download_photo(self, message_media_photo, file_path, add_extension=False,
                       progress_callback=None):
        """Downloads MessageMediaPhoto's largest size into the desired
           file_path, optionally finding its extension automatically
           The progress_callback should be a callback function which takes two parameters,
           uploaded size (in bytes) and total file size (in bytes).
           This will be called every time a part is downloaded"""

        # Determine the photo and its largest size
        photo = message_media_photo.photo
        largest_size = photo.sizes[-1]
        file_size = largest_size.size
        largest_size = largest_size.location

        if add_extension:
            file_path += get_extension(message_media_photo)

        # Download the media with the largest size input file location
        self.download_file_loc(InputFileLocation(volume_id=largest_size.volume_id,
                                                 local_id=largest_size.local_id,
                                                 secret=largest_size.secret),
                               file_path, file_size=file_size, progress_callback=progress_callback)
        return file_path

    def download_document(self, message_media_document, file_path=None, add_extension=True,
                          progress_callback=None):
        """Downloads the given MessageMediaDocument into the desired
           file_path, optionally finding its extension automatically.
           If no file_path is given, it will try to be guessed from the document
           The progress_callback should be a callback function which takes two parameters,
           uploaded size (in bytes) and total file size (in bytes).
           This will be called every time a part is downloaded"""
        document = message_media_document.document
        file_size = document.size

        # If no file path was given, try to guess it from the attributes
        if file_path is None:
            for attr in document.attributes:
                if type(attr) == DocumentAttributeFilename:
                    file_path = attr.file_name
                    break  # This attribute has higher preference

                elif type(attr) == DocumentAttributeAudio:
                    file_path = '{} - {}'.format(attr.performer, attr.title)

            if file_path is None:
                print('Could not determine a filename for the document')

        if add_extension:
            file_path += get_extension(document.mime_type)

        self.download_file_loc(InputDocumentFileLocation(id=document.id,
                                                         access_hash=document.access_hash,
                                                         version=document.version),
                               file_path, file_size=file_size, progress_callback=progress_callback)
        return file_path

    @staticmethod
    def download_contact(message_media_contact, file_path, add_extension=True):
        """Downloads a media contact using the vCard 4.0 format"""

        first_name = message_media_contact.first_name
        last_name = message_media_contact.last_name
        phone_number = message_media_contact.phone_number

        # The only way we can save a contact in an understandable
        # way by phones is by using the .vCard format
        if add_extension:
            file_path += '.vcard'

        # Ensure that we'll be able to download the contact
        utils.ensure_parent_dir_exists(file_path)

        with open(file_path, 'w', encoding='utf-8') as file:
            file.write('BEGIN:VCARD\n')
            file.write('VERSION:4.0\n')
            file.write('N:{};{};;;\n'.format(first_name, last_name if last_name else ''))
            file.write('FN:{}\n'.format(' '.join((first_name, last_name))))
            file.write('TEL;TYPE=cell;VALUE=uri:tel:+{}\n'.format(phone_number))
            file.write('END:VCARD\n')

        return file_path

    def download_file_loc(self, input_location, file_path, part_size_kb=64,
                          file_size=None, progress_callback=None):
        """Downloads media from the given input_file_location to the specified file_path.
           If a progress_callback function is given, it will be called taking two
           arguments (downloaded bytes count and total file size)"""

        if not part_size_kb:
            if not file_size:
                raise ValueError('A part size value must be provided')
            else:
                part_size_kb = get_appropiate_part_size(file_size)

        part_size = int(part_size_kb * 1024)
        if part_size % 1024 != 0:
            raise ValueError('The part size must be evenly divisible by 1024')

        # Ensure that we'll be able to download the media
        utils.ensure_parent_dir_exists(file_path)

        # Start with an offset index of 0
        offset_index = 0
        with open(file_path, 'wb') as file:
            while True:
                # The current offset equals the offset_index multiplied by the part size
                offset = offset_index * part_size
                result = self.invoke(GetFileRequest(input_location, offset, part_size))
                offset_index += 1

                # If we have received no data (0 bytes), the file is over
                # So there is nothing left to download and write
                if not result.bytes:
                    return result.type  # Return some extra information

                file.write(result.bytes)
                if progress_callback:
                    progress_callback(file.tell(), file_size)

    # endregion

    # endregion

    # region Updates handling

    def add_update_handler(self, handler):
        """Adds an update handler (a function which takes a TLObject,
          an update, as its parameter) and listens for updates"""
        if not self.signed_in:
            raise ValueError("You cannot add update handlers until you've signed in.")

        self.sender.add_update_handler(handler)

    def remove_update_handler(self, handler):
        self.sender.remove_update_handler(handler)

    # endregion



from telethon.tl.mtproto_request import MTProtoRequest


class ChangePhoneRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    account.changePhone#70c32edb phone_number:string phone_code_hash:string phone_code:string = User"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x70c32edb

    def __init__(self, phone_number, phone_code_hash, phone_code):
        """
        :param phone_number: Telegram type: «string».
        :param phone_code_hash: Telegram type: «string».
        :param phone_code: Telegram type: «string».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.phone_number = phone_number
        self.phone_code_hash = phone_code_hash
        self.phone_code = phone_code

    def on_send(self, writer):
        writer.write_int(ChangePhoneRequest.constructor_id, signed=False)
        writer.tgwrite_string(self.phone_number)
        writer.tgwrite_string(self.phone_code_hash)
        writer.tgwrite_string(self.phone_code)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ChangePhoneRequest(None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'account.changePhone#70c32edb phone_number:string phone_code_hash:string phone_code:string = User'

    def __str__(self):
        return '(account.changePhone (ID: 0x70c32edb) = (phone_number={}, phone_code_hash={}, phone_code={}))'.format(str(self.phone_number), str(self.phone_code_hash), str(self.phone_code))



from telethon.tl.types.storage.file_unknown import FileUnknown
from telethon.tl.types.storage.file_jpeg import FileJpeg
from telethon.tl.types.storage.file_gif import FileGif
from telethon.tl.types.storage.file_png import FilePng
from telethon.tl.types.storage.file_pdf import FilePdf
from telethon.tl.types.storage.file_mp3 import FileMp3
from telethon.tl.types.storage.file_mov import FileMov
from telethon.tl.types.storage.file_partial import FilePartial
from telethon.tl.types.storage.file_mp4 import FileMp4
from telethon.tl.types.storage.file_webp import FileWebp



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateServiceNotification(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateServiceNotification#382dd3e4 type:string message:string media:MessageMedia popup:Bool = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x382dd3e4

    def __init__(self, type, message, media, popup):
        """
        :param type: Telegram type: «string».
        :param message: Telegram type: «string».
        :param media: Telegram type: «MessageMedia».
        :param popup: Telegram type: «Bool».
        """
        super().__init__()

        self.type = type
        self.message = message
        self.media = media
        self.popup = popup

    def on_send(self, writer):
        writer.write_int(UpdateServiceNotification.constructor_id, signed=False)
        writer.tgwrite_string(self.type)
        writer.tgwrite_string(self.message)
        self.media.on_send(writer)
        writer.tgwrite_bool(self.popup)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateServiceNotification(None, None, None, None)

    def on_response(self, reader):
        self.type = reader.tgread_string()
        self.message = reader.tgread_string()
        self.media = reader.tgread_object()
        self.popup = reader.tgread_bool()

    def __repr__(self):
        return 'updateServiceNotification#382dd3e4 type:string message:string media:MessageMedia popup:Bool = Update'

    def __str__(self):
        return '(updateServiceNotification (ID: 0x382dd3e4) = (type={}, message={}, media={}, popup={}))'.format(str(self.type), str(self.message), str(self.media), str(self.popup))



from telethon.tl.mtproto_request import MTProtoRequest


class ChatParticipants(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    chatParticipants#3f460fed chat_id:int participants:Vector<ChatParticipant> version:int = ChatParticipants"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x3f460fed

    def __init__(self, chat_id, participants, version):
        """
        :param chat_id: Telegram type: «int».
        :param participants: Telegram type: «ChatParticipant». Must be a list.
        :param version: Telegram type: «int».
        """
        super().__init__()

        self.chat_id = chat_id
        self.participants = participants
        self.version = version

    def on_send(self, writer):
        writer.write_int(ChatParticipants.constructor_id, signed=False)
        writer.write_int(self.chat_id)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.participants))
        for participants_item in self.participants:
            participants_item.on_send(writer)

        writer.write_int(self.version)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ChatParticipants(None, None, None)

    def on_response(self, reader):
        self.chat_id = reader.read_int()
        reader.read_int()  # Vector's constructor ID
        self.participants = []  # Initialize an empty list
        participants_len = reader.read_int()
        for _ in range(participants_len):
            participants_item = reader.tgread_object()
            self.participants.append(participants_item)

        self.version = reader.read_int()

    def __repr__(self):
        return 'chatParticipants#3f460fed chat_id:int participants:Vector<ChatParticipant> version:int = ChatParticipants'

    def __str__(self):
        return '(chatParticipants (ID: 0x3f460fed) = (chat_id={}, participants={}, version={}))'.format(str(self.chat_id), None if not self.participants else [str(_) for _ in self.participants], str(self.version))



from telethon.tl.mtproto_request import MTProtoRequest


class CreateChannelRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    channels.createChannel#f4893d7f flags:None broadcast:flags.0?true megagroup:flags.1?true title:string about:string = Updates"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xf4893d7f

    def __init__(self, title, about, broadcast=None, megagroup=None):
        """
        :param broadcast: Telegram type: «true».
        :param megagroup: Telegram type: «true».
        :param title: Telegram type: «string».
        :param about: Telegram type: «string».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.broadcast = broadcast
        self.megagroup = megagroup
        self.title = title
        self.about = about

    def on_send(self, writer):
        writer.write_int(CreateChannelRequest.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.broadcast else 0
        flags |= (1 << 1) if self.megagroup else 0
        writer.write_int(flags)

        writer.tgwrite_string(self.title)
        writer.tgwrite_string(self.about)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return CreateChannelRequest(None, None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'channels.createChannel#f4893d7f flags:None broadcast:flags.0?true megagroup:flags.1?true title:string about:string = Updates'

    def __str__(self):
        return '(channels.createChannel (ID: 0xf4893d7f) = (broadcast={}, megagroup={}, title={}, about={}))'.format(str(self.broadcast), str(self.megagroup), str(self.title), str(self.about))



from telethon.tl.mtproto_request import MTProtoRequest


class GetTopPeersRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    contacts.getTopPeers#d4982db5 flags:None correspondents:flags.0?true bots_pm:flags.1?true bots_inline:flags.2?true groups:flags.10?true channels:flags.15?true offset:int limit:int hash:int = contacts.TopPeers"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xd4982db5

    def __init__(self, offset, limit, hash, correspondents=None, bots_pm=None, bots_inline=None, groups=None, channels=None):
        """
        :param correspondents: Telegram type: «true».
        :param bots_pm: Telegram type: «true».
        :param bots_inline: Telegram type: «true».
        :param groups: Telegram type: «true».
        :param channels: Telegram type: «true».
        :param offset: Telegram type: «int».
        :param limit: Telegram type: «int».
        :param hash: Telegram type: «int».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.correspondents = correspondents
        self.bots_pm = bots_pm
        self.bots_inline = bots_inline
        self.groups = groups
        self.channels = channels
        self.offset = offset
        self.limit = limit
        self.hash = hash

    def on_send(self, writer):
        writer.write_int(GetTopPeersRequest.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.correspondents else 0
        flags |= (1 << 1) if self.bots_pm else 0
        flags |= (1 << 2) if self.bots_inline else 0
        flags |= (1 << 10) if self.groups else 0
        flags |= (1 << 15) if self.channels else 0
        writer.write_int(flags)

        writer.write_int(self.offset)
        writer.write_int(self.limit)
        writer.write_int(self.hash)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return GetTopPeersRequest(None, None, None, None, None, None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'contacts.getTopPeers#d4982db5 flags:None correspondents:flags.0?true bots_pm:flags.1?true bots_inline:flags.2?true groups:flags.10?true channels:flags.15?true offset:int limit:int hash:int = contacts.TopPeers'

    def __str__(self):
        return '(contacts.getTopPeers (ID: 0xd4982db5) = (correspondents={}, bots_pm={}, bots_inline={}, groups={}, channels={}, offset={}, limit={}, hash={}))'.format(str(self.correspondents), str(self.bots_pm), str(self.bots_inline), str(self.groups), str(self.channels), str(self.offset), str(self.limit), str(self.hash))



from telethon.tl.mtproto_request import MTProtoRequest


class ChatInviteAlready(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    chatInviteAlready#5a686d7c chat:Chat = ChatInvite"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x5a686d7c

    def __init__(self, chat):
        """
        :param chat: Telegram type: «Chat».
        """
        super().__init__()

        self.chat = chat

    def on_send(self, writer):
        writer.write_int(ChatInviteAlready.constructor_id, signed=False)
        self.chat.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ChatInviteAlready(None)

    def on_response(self, reader):
        self.chat = reader.tgread_object()

    def __repr__(self):
        return 'chatInviteAlready#5a686d7c chat:Chat = ChatInvite'

    def __str__(self):
        return '(chatInviteAlready (ID: 0x5a686d7c) = (chat={}))'.format(str(self.chat))



from telethon.tl.mtproto_request import MTProtoRequest


class DifferenceEmpty(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updates.differenceEmpty#5d75a138 date:date seq:int = updates.Difference"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x5d75a138

    def __init__(self, date, seq):
        """
        :param date: Telegram type: «date».
        :param seq: Telegram type: «int».
        """
        super().__init__()

        self.date = date
        self.seq = seq

    def on_send(self, writer):
        writer.write_int(DifferenceEmpty.constructor_id, signed=False)
        writer.tgwrite_date(self.date)
        writer.write_int(self.seq)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return DifferenceEmpty(None, None)

    def on_response(self, reader):
        self.date = reader.tgread_date()
        self.seq = reader.read_int()

    def __repr__(self):
        return 'updates.differenceEmpty#5d75a138 date:date seq:int = updates.Difference'

    def __str__(self):
        return '(updates.differenceEmpty (ID: 0x5d75a138) = (date={}, seq={}))'.format(str(self.date), str(self.seq))



from telethon.tl.mtproto_request import MTProtoRequest


class MsgsAllInfo(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    msgs_all_info#8cc0d131 msg_ids:Vector<long> info:string = MsgsAllInfo"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x8cc0d131

    def __init__(self, msg_ids, info):
        """
        :param msg_ids: Telegram type: «long». Must be a list.
        :param info: Telegram type: «string».
        """
        super().__init__()

        self.msg_ids = msg_ids
        self.info = info

    def on_send(self, writer):
        writer.write_int(MsgsAllInfo.constructor_id, signed=False)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.msg_ids))
        for msg_ids_item in self.msg_ids:
            writer.write_long(msg_ids_item)

        writer.tgwrite_string(self.info)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return MsgsAllInfo(None, None)

    def on_response(self, reader):
        reader.read_int()  # Vector's constructor ID
        self.msg_ids = []  # Initialize an empty list
        msg_ids_len = reader.read_int()
        for _ in range(msg_ids_len):
            msg_ids_item = reader.read_long()
            self.msg_ids.append(msg_ids_item)

        self.info = reader.tgread_string()

    def __repr__(self):
        return 'msgs_all_info#8cc0d131 msg_ids:Vector<long> info:string = MsgsAllInfo'

    def __str__(self):
        return '(msgs_all_info (ID: 0x8cc0d131) = (msg_ids={}, info={}))'.format(None if not self.msg_ids else [str(_) for _ in self.msg_ids], str(self.info))



from telethon.tl.mtproto_request import MTProtoRequest


class EditChatTitleRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.editChatTitle#dc452855 chat_id:int title:string = Updates"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xdc452855

    def __init__(self, chat_id, title):
        """
        :param chat_id: Telegram type: «int».
        :param title: Telegram type: «string».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.chat_id = chat_id
        self.title = title

    def on_send(self, writer):
        writer.write_int(EditChatTitleRequest.constructor_id, signed=False)
        writer.write_int(self.chat_id)
        writer.tgwrite_string(self.title)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return EditChatTitleRequest(None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.editChatTitle#dc452855 chat_id:int title:string = Updates'

    def __str__(self):
        return '(messages.editChatTitle (ID: 0xdc452855) = (chat_id={}, title={}))'.format(str(self.chat_id), str(self.title))



from telethon.tl.mtproto_request import MTProtoRequest


class DocumentAttributeFilename(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    documentAttributeFilename#15590068 file_name:string = DocumentAttribute"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x15590068

    def __init__(self, file_name):
        """
        :param file_name: Telegram type: «string».
        """
        super().__init__()

        self.file_name = file_name

    def on_send(self, writer):
        writer.write_int(DocumentAttributeFilename.constructor_id, signed=False)
        writer.tgwrite_string(self.file_name)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return DocumentAttributeFilename(None)

    def on_response(self, reader):
        self.file_name = reader.tgread_string()

    def __repr__(self):
        return 'documentAttributeFilename#15590068 file_name:string = DocumentAttribute'

    def __str__(self):
        return '(documentAttributeFilename (ID: 0x15590068) = (file_name={}))'.format(str(self.file_name))



from telethon.tl.functions.updates.get_state import GetStateRequest
from telethon.tl.functions.updates.get_difference import GetDifferenceRequest
from telethon.tl.functions.updates.get_channel_difference import GetChannelDifferenceRequest



from telethon.tl.functions.account.register_device import RegisterDeviceRequest
from telethon.tl.functions.account.unregister_device import UnregisterDeviceRequest
from telethon.tl.functions.account.update_notify_settings import UpdateNotifySettingsRequest
from telethon.tl.functions.account.get_notify_settings import GetNotifySettingsRequest
from telethon.tl.functions.account.reset_notify_settings import ResetNotifySettingsRequest
from telethon.tl.functions.account.update_profile import UpdateProfileRequest
from telethon.tl.functions.account.update_status import UpdateStatusRequest
from telethon.tl.functions.account.get_wall_papers import GetWallPapersRequest
from telethon.tl.functions.account.report_peer import ReportPeerRequest
from telethon.tl.functions.account.check_username import CheckUsernameRequest
from telethon.tl.functions.account.update_username import UpdateUsernameRequest
from telethon.tl.functions.account.get_privacy import GetPrivacyRequest
from telethon.tl.functions.account.set_privacy import SetPrivacyRequest
from telethon.tl.functions.account.delete_account import DeleteAccountRequest
from telethon.tl.functions.account.get_account_ttl import GetAccountTTLRequest
from telethon.tl.functions.account.set_account_ttl import SetAccountTTLRequest
from telethon.tl.functions.account.send_change_phone_code import SendChangePhoneCodeRequest
from telethon.tl.functions.account.change_phone import ChangePhoneRequest
from telethon.tl.functions.account.update_device_locked import UpdateDeviceLockedRequest
from telethon.tl.functions.account.get_authorizations import GetAuthorizationsRequest
from telethon.tl.functions.account.reset_authorization import ResetAuthorizationRequest
from telethon.tl.functions.account.get_password import GetPasswordRequest
from telethon.tl.functions.account.get_password_settings import GetPasswordSettingsRequest
from telethon.tl.functions.account.update_password_settings import UpdatePasswordSettingsRequest
from telethon.tl.functions.account.send_confirm_phone_code import SendConfirmPhoneCodeRequest
from telethon.tl.functions.account.confirm_phone import ConfirmPhoneRequest



from telethon.tl.mtproto_request import MTProtoRequest


class ServerDHParamsOk(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    server_DH_params_ok#d0e8075c nonce:int128 server_nonce:int128 encrypted_answer:string = Server_DH_Params"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xd0e8075c

    def __init__(self, nonce, server_nonce, encrypted_answer):
        """
        :param nonce: Telegram type: «int128».
        :param server_nonce: Telegram type: «int128».
        :param encrypted_answer: Telegram type: «string».
        """
        super().__init__()

        self.nonce = nonce
        self.server_nonce = server_nonce
        self.encrypted_answer = encrypted_answer

    def on_send(self, writer):
        writer.write_int(ServerDHParamsOk.constructor_id, signed=False)
        writer.write_large_int(self.nonce, bits=128)
        writer.write_large_int(self.server_nonce, bits=128)
        writer.tgwrite_string(self.encrypted_answer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ServerDHParamsOk(None, None, None)

    def on_response(self, reader):
        self.nonce = reader.read_large_int(bits=128)
        self.server_nonce = reader.read_large_int(bits=128)
        self.encrypted_answer = reader.tgread_string()

    def __repr__(self):
        return 'server_DH_params_ok#d0e8075c nonce:int128 server_nonce:int128 encrypted_answer:string = Server_DH_Params'

    def __str__(self):
        return '(server_DH_params_ok (ID: 0xd0e8075c) = (nonce={}, server_nonce={}, encrypted_answer={}))'.format(str(self.nonce), str(self.server_nonce), str(self.encrypted_answer))



from telethon.tl.mtproto_request import MTProtoRequest


class SendMediaRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.sendMedia#c8f16791 flags:None silent:flags.5?true background:flags.6?true clear_draft:flags.7?true peer:InputPeer reply_to_msg_id:flags.0?int media:InputMedia random_id:long reply_markup:flags.2?ReplyMarkup = Updates"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xc8f16791

    def __init__(self, peer, media, random_id, silent=None, background=None, clear_draft=None, reply_to_msg_id=None, reply_markup=None):
        """
        :param silent: Telegram type: «true».
        :param background: Telegram type: «true».
        :param clear_draft: Telegram type: «true».
        :param peer: Telegram type: «InputPeer».
        :param reply_to_msg_id: Telegram type: «int».
        :param media: Telegram type: «InputMedia».
        :param random_id: Telegram type: «long».
        :param reply_markup: Telegram type: «ReplyMarkup».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.silent = silent
        self.background = background
        self.clear_draft = clear_draft
        self.peer = peer
        self.reply_to_msg_id = reply_to_msg_id
        self.media = media
        self.random_id = random_id
        self.reply_markup = reply_markup

    def on_send(self, writer):
        writer.write_int(SendMediaRequest.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 5) if self.silent else 0
        flags |= (1 << 6) if self.background else 0
        flags |= (1 << 7) if self.clear_draft else 0
        flags |= (1 << 0) if self.reply_to_msg_id else 0
        flags |= (1 << 2) if self.reply_markup else 0
        writer.write_int(flags)

        self.peer.on_send(writer)
        if self.reply_to_msg_id:
            writer.write_int(self.reply_to_msg_id)

        self.media.on_send(writer)
        writer.write_long(self.random_id)
        if self.reply_markup:
            self.reply_markup.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return SendMediaRequest(None, None, None, None, None, None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.sendMedia#c8f16791 flags:None silent:flags.5?true background:flags.6?true clear_draft:flags.7?true peer:InputPeer reply_to_msg_id:flags.0?int media:InputMedia random_id:long reply_markup:flags.2?ReplyMarkup = Updates'

    def __str__(self):
        return '(messages.sendMedia (ID: 0xc8f16791) = (silent={}, background={}, clear_draft={}, peer={}, reply_to_msg_id={}, media={}, random_id={}, reply_markup={}))'.format(str(self.silent), str(self.background), str(self.clear_draft), str(self.peer), str(self.reply_to_msg_id), str(self.media), str(self.random_id), str(self.reply_markup))



from telethon.tl.mtproto_request import MTProtoRequest


class ReplyKeyboardHide(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    replyKeyboardHide#a03e5b85 flags:None selective:flags.2?true = ReplyMarkup"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xa03e5b85

    def __init__(self, selective=None):
        """
        :param selective: Telegram type: «true».
        """
        super().__init__()

        self.selective = selective

    def on_send(self, writer):
        writer.write_int(ReplyKeyboardHide.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 2) if self.selective else 0
        writer.write_int(flags)


    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ReplyKeyboardHide(None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 2)) != 0:
            self.selective = True  # Arbitrary not-None value, no need to read since it is a flag

    def __repr__(self):
        return 'replyKeyboardHide#a03e5b85 flags:None selective:flags.2?true = ReplyMarkup'

    def __str__(self):
        return '(replyKeyboardHide (ID: 0xa03e5b85) = (selective={}))'.format(str(self.selective))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateNewStickerSet(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateNewStickerSet#688a30aa stickerset:messages.StickerSet = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x688a30aa

    def __init__(self, stickerset):
        """
        :param stickerset: Telegram type: «messages.StickerSet».
        """
        super().__init__()

        self.stickerset = stickerset

    def on_send(self, writer):
        writer.write_int(UpdateNewStickerSet.constructor_id, signed=False)
        self.stickerset.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateNewStickerSet(None)

    def on_response(self, reader):
        self.stickerset = reader.tgread_object()

    def __repr__(self):
        return 'updateNewStickerSet#688a30aa stickerset:messages.StickerSet = Update'

    def __str__(self):
        return '(updateNewStickerSet (ID: 0x688a30aa) = (stickerset={}))'.format(str(self.stickerset))



from telethon.tl.functions.upload.save_file_part import SaveFilePartRequest
from telethon.tl.functions.upload.get_file import GetFileRequest
from telethon.tl.functions.upload.save_big_file_part import SaveBigFilePartRequest



from telethon.tl.types.help.app_update import AppUpdate
from telethon.tl.types.help.no_app_update import NoAppUpdate
from telethon.tl.types.help.invite_text import InviteText
from telethon.tl.types.help.support import Support
from telethon.tl.types.help.app_changelog_empty import AppChangelogEmpty
from telethon.tl.types.help.app_changelog import AppChangelog
from telethon.tl.types.help.terms_of_service import TermsOfService



from telethon.tl.mtproto_request import MTProtoRequest


class ReqDHParamsRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    req_DH_params#d712e4be nonce:int128 server_nonce:int128 p:string q:string public_key_fingerprint:long encrypted_data:string = Server_DH_Params"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xd712e4be

    def __init__(self, nonce, server_nonce, p, q, public_key_fingerprint, encrypted_data):
        """
        :param nonce: Telegram type: «int128».
        :param server_nonce: Telegram type: «int128».
        :param p: Telegram type: «string».
        :param q: Telegram type: «string».
        :param public_key_fingerprint: Telegram type: «long».
        :param encrypted_data: Telegram type: «string».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.nonce = nonce
        self.server_nonce = server_nonce
        self.p = p
        self.q = q
        self.public_key_fingerprint = public_key_fingerprint
        self.encrypted_data = encrypted_data

    def on_send(self, writer):
        writer.write_int(ReqDHParamsRequest.constructor_id, signed=False)
        writer.write_large_int(self.nonce, bits=128)
        writer.write_large_int(self.server_nonce, bits=128)
        writer.tgwrite_string(self.p)
        writer.tgwrite_string(self.q)
        writer.write_long(self.public_key_fingerprint)
        writer.tgwrite_string(self.encrypted_data)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ReqDHParamsRequest(None, None, None, None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'req_DH_params#d712e4be nonce:int128 server_nonce:int128 p:string q:string public_key_fingerprint:long encrypted_data:string = Server_DH_Params'

    def __str__(self):
        return '(req_DH_params (ID: 0xd712e4be) = (nonce={}, server_nonce={}, p={}, q={}, public_key_fingerprint={}, encrypted_data={}))'.format(str(self.nonce), str(self.server_nonce), str(self.p), str(self.q), str(self.public_key_fingerprint), str(self.encrypted_data))



from telethon.tl.mtproto_request import MTProtoRequest


class SendMessageUploadAudioAction(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    sendMessageUploadAudioAction#f351d7ab progress:int = SendMessageAction"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xf351d7ab

    def __init__(self, progress):
        """
        :param progress: Telegram type: «int».
        """
        super().__init__()

        self.progress = progress

    def on_send(self, writer):
        writer.write_int(SendMessageUploadAudioAction.constructor_id, signed=False)
        writer.write_int(self.progress)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return SendMessageUploadAudioAction(None)

    def on_response(self, reader):
        self.progress = reader.read_int()

    def __repr__(self):
        return 'sendMessageUploadAudioAction#f351d7ab progress:int = SendMessageAction'

    def __str__(self):
        return '(sendMessageUploadAudioAction (ID: 0xf351d7ab) = (progress={}))'.format(str(self.progress))



# Python rough implementation of a C# TCP client
import socket
import time
from datetime import datetime, timedelta
from threading import Lock

from telethon.errors import ReadCancelledError
from telethon.utils import BinaryWriter


class TcpClient:
    def __init__(self):
        self.connected = False
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        # Support for multi-threading advantages and safety
        self.cancelled = False  # Has the read operation been cancelled?
        self.delay = 0.1  # Read delay when there was no data available
        self.lock = Lock()

    def connect(self, ip, port):
        """Connects to the specified IP and port number"""
        self.socket.connect((ip, port))
        self.connected = True

    def close(self):
        """Closes the connection"""
        self.socket.close()
        self.connected = False
        self.socket.setblocking(True)

    def write(self, data):
        """Writes (sends) the specified bytes to the connected peer"""

        # Ensure that only one thread can send data at once
        with self.lock:
            # Set blocking so it doesn't error
            self.socket.setblocking(True)
            self.socket.sendall(data)

    def read(self, buffer_size, timeout=timedelta(seconds=5)):
        """Reads (receives) the specified bytes from the connected peer.
           A timeout can be specified, which will cancel the operation if no data
           has been read in the specified time. If data was read and it's waiting
           for more, the timeout will NOT cancel the operation. Set to None for no timeout"""

        # Ensure that only one thread can receive data at once
        with self.lock:
            # Ensure it is not cancelled at first, so we can enter the loop
            self.cancelled = False

            # Set non-blocking so it can be cancelled
            self.socket.setblocking(False)

            # Set the starting time so we can calculate whether the timeout should fire
            if timeout:
                start_time = datetime.now()

            with BinaryWriter() as writer:
                while writer.written_count < buffer_size:
                    # Only do cancel if no data was read yet
                    # Otherwise, carry on reading and finish
                    if self.cancelled and writer.written_count == 0:
                        raise ReadCancelledError()

                    try:
                        # When receiving from the socket, we may not receive all the data at once
                        # This is why we need to keep checking to make sure that we receive it all
                        left_count = buffer_size - writer.written_count
                        partial = self.socket.recv(left_count)
                        writer.write(partial)

                    except BlockingIOError:
                        # There was no data available for us to read. Sleep a bit
                        time.sleep(self.delay)

                        # Check if the timeout finished
                        if timeout:
                            time_passed = datetime.now() - start_time
                            if time_passed > timeout:
                                raise TimeoutError('The read operation exceeded the timeout.')

                # If everything went fine, return the read bytes
                return writer.get_bytes()

    def cancel_read(self):
        """Cancels the read operation IF it hasn't yet
           started, raising a ReadCancelledError"""
        self.cancelled = True



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateMessageID(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateMessageID#4e90bfd6 id:int random_id:long = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x4e90bfd6

    def __init__(self, id, random_id):
        """
        :param id: Telegram type: «int».
        :param random_id: Telegram type: «long».
        """
        super().__init__()

        self.id = id
        self.random_id = random_id

    def on_send(self, writer):
        writer.write_int(UpdateMessageID.constructor_id, signed=False)
        writer.write_int(self.id)
        writer.write_long(self.random_id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateMessageID(None, None)

    def on_response(self, reader):
        self.id = reader.read_int()
        self.random_id = reader.read_long()

    def __repr__(self):
        return 'updateMessageID#4e90bfd6 id:int random_id:long = Update'

    def __str__(self):
        return '(updateMessageID (ID: 0x4e90bfd6) = (id={}, random_id={}))'.format(str(self.id), str(self.random_id))
